@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -14,10 +16,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from nova import flags
 from nova import manager
-from nova.network import driver
 from nova import utils
 
+FLAGS = flags.FLAGS
+
 
 class MetadataManager(manager.Manager):
     """Metadata Manager.
@@ -27,10 +31,12 @@ class MetadataManager(manager.Manager):
     """
     def __init__(self, *args, **kwargs):
         super(MetadataManager, self).__init__(*args, **kwargs)
+        self.network_driver = utils.import_object(FLAGS.network_driver)
 
-        if not utils.is_neutron():
-            # NOTE(mikal): we only add iptables rules if we're running
-            # under nova-network. This code should go away when the
-            # deprecation of nova-network is complete.
-            self.network_driver = driver.load_network_driver()
-            self.network_driver.metadata_accept()
+    def init_host(self):
+        """Perform any initialization.
+
+        Currently, we only add an iptables filter rule for the metadata
+        service.
+        """
+        self.network_driver.metadata_accept()
