@@ -24,9 +24,7 @@ Nova authentication management
 """
 
 import os
-import shutil
 import string  # pylint: disable=W0402
-import tempfile
 import uuid
 import zipfile
 
@@ -36,53 +34,70 @@ from nova import db
 from nova import exception
 from nova import flags
 from nova import log as logging
+from nova.openstack.common import cfg
 from nova import utils
 from nova.auth import signer
 
 
+auth_opts = [
+    cfg.ListOpt('allowed_roles',
+                default=[
+                  'cloudadmin',
+                  'itsec',
+                  'sysadmin',
+                  'netadmin',
+                  'developer'
+                  ],
+                help='Allowed roles for project'),
+
+    # NOTE(vish): a user with one of these roles will be a superuser and
+    #             have access to all api commands
+    cfg.ListOpt('superuser_roles',
+                default=['cloudadmin'],
+                help='Roles that ignore authorization checking completely'),
+
+    # NOTE(vish): a user with one of these roles will have it for every
+    #             project, even if he or she is not a member of the project
+    cfg.ListOpt('global_roles',
+                default=['cloudadmin', 'itsec'],
+                help='Roles that apply to all projects'),
+
+    cfg.StrOpt('credentials_template',
+               default='$pybasedir/nova/auth/novarc.template',
+               help='Template for creating users rc file'),
+    cfg.StrOpt('vpn_client_template',
+               default='$pybasedir/nova/cloudpipe/client.ovpn.template',
+               help='Template for creating users vpn file'),
+    cfg.StrOpt('credential_vpn_file',
+               default='nova-vpn.conf',
+               help='Filename of certificate in credentials zip'),
+    cfg.StrOpt('credential_key_file',
+               default='pk.pem',
+               help='Filename of private key in credentials zip'),
+    cfg.StrOpt('credential_cert_file',
+               default='cert.pem',
+               help='Filename of certificate in credentials zip'),
+    cfg.StrOpt('credential_rc_file',
+               default='%src',
+               help='Filename of rc in credentials zip %s will be replaced by '
+                    'name of the region (nova by default)'),
+    cfg.StrOpt('auth_driver',
+               default='nova.auth.dbdriver.DbDriver',
+               help='Driver that auth manager uses'),
+    ]
+
 FLAGS = flags.FLAGS
-flags.DEFINE_bool('use_deprecated_auth',
-                  False,
-                  'This flag must be set to use old style auth')
+FLAGS.register_opts(auth_opts)
 
-flags.DEFINE_list('allowed_roles',
-                  ['cloudadmin', 'itsec', 'sysadmin', 'netadmin', 'developer'],
-                  'Allowed roles for project')
-# NOTE(vish): a user with one of these roles will be a superuser and
-#             have access to all api commands
-flags.DEFINE_list('superuser_roles', ['cloudadmin'],
-                  'Roles that ignore authorization checking completely')
+flags.DECLARE('osapi_compute_listen_port', 'nova.service')
 
-# NOTE(vish): a user with one of these roles will have it for every
-#             project, even if he or she is not a member of the project
-flags.DEFINE_list('global_roles', ['cloudadmin', 'itsec'],
-                  'Roles that apply to all projects')
-
-flags.DEFINE_string('credentials_template',
-                    utils.abspath('auth/novarc.template'),
-                    'Template for creating users rc file')
-flags.DEFINE_string('vpn_client_template',
-                    utils.abspath('cloudpipe/client.ovpn.template'),
-                    'Template for creating users vpn file')
-flags.DEFINE_string('credential_vpn_file', 'nova-vpn.conf',
-                    'Filename of certificate in credentials zip')
-flags.DEFINE_string('credential_key_file', 'pk.pem',
-                    'Filename of private key in credentials zip')
-flags.DEFINE_string('credential_cert_file', 'cert.pem',
-                    'Filename of certificate in credentials zip')
-flags.DEFINE_string('credential_rc_file', '%src',
-                    'Filename of rc in credentials zip, %s will be '
-                    'replaced by name of the region (nova by default)')
-flags.DEFINE_string('auth_driver', 'nova.auth.dbdriver.DbDriver',
-                    'Driver that auth manager uses')
-
-LOG = logging.getLogger('nova.auth.manager')
+LOG = logging.getLogger(__name__)
 
 
 if FLAGS.memcached_servers:
     import memcache
 else:
-    from nova import fakememcache as memcache
+    from nova.common import memorycache as memcache
 
 
 class AuthBase(object):
@@ -99,7 +114,7 @@ class AuthBase(object):
 
         This method will return the id of the object if the object
         is of this class, otherwise it will return the original object.
-        This allows methods to accept objects or ids as paramaters.
+        This allows methods to accept objects or ids as parameters.
         """
         if isinstance(obj, cls):
             return obj.id
@@ -111,11 +126,17 @@ class User(AuthBase):
     """Object representing a user
 
     The following attributes are defined:
-    :id:       A system identifier for the user.  A string (for LDAP)
-    :name:     The user name, potentially in some more friendly format
-    :access:   The 'username' for EC2 authentication
-    :secret:   The 'password' for EC2 authenticatoin
-    :admin:    ???
+
+    ``id``
+      A system identifier for the user.  A string (for LDAP)
+    ``name``
+      The user name, potentially in some more friendly format
+    ``access``
+      The 'username' for EC2 authentication
+    ``secret``
+      The 'password' for EC2 authenticatoin
+    ``admin``
+      ???
     """
 
     def __init__(self, id, name, access, secret, admin):
@@ -149,11 +170,7 @@ class User(AuthBase):
         return AuthManager().is_project_manager(self, project)
 
     def __repr__(self):
-        return "User('%s', '%s', '%s', '%s', %s)" % (self.id,
-                                                     self.name,
-                                                     self.access,
-                                                     self.secret,
-                                                     self.admin)
+        return "User('%s', '%s')" % (self.id, self.name)
 
 
 class Project(AuthBase):
@@ -200,9 +217,7 @@ class Project(AuthBase):
         return AuthManager().get_credentials(user, self)
 
     def __repr__(self):
-        return "Project('%s', '%s', '%s', '%s', %s)" % \
-            (self.id, self.name, self.project_manager_id, self.description,
-             self.member_ids)
+        return "Project('%s', '%s')" % (self.id, self.name)
 
 
 class AuthManager(object):
@@ -247,35 +262,35 @@ class AuthManager(object):
         a project with the same name as the user. This way, older tools
         that have no project knowledge will still work.
 
-        @type access: str
-        @param access: Access key for user in the form "access:project".
+        :type access: str
+        :param access: Access key for user in the form "access:project".
 
-        @type signature: str
-        @param signature: Signature of the request.
+        :type signature: str
+        :param signature: Signature of the request.
 
-        @type params: list of str
-        @param params: Web paramaters used for the signature.
+        :type params: list of str
+        :param params: Web paramaters used for the signature.
 
-        @type verb: str
-        @param verb: Web request verb ('GET' or 'POST').
+        :type verb: str
+        :param verb: Web request verb ('GET' or 'POST').
 
-        @type server_string: str
-        @param server_string: Web request server string.
+        :type server_string: str
+        :param server_string: Web request server string.
 
-        @type path: str
-        @param path: Web request path.
+        :type path: str
+        :param path: Web request path.
 
-        @type check_type: str
-        @param check_type: Type of signature to check. 'ec2' for EC2, 's3' for
+        :type check_type: str
+        :param check_type: Type of signature to check. 'ec2' for EC2, 's3' for
                            S3. Any other value will cause signature not to be
                            checked.
 
-        @type headers: list
-        @param headers: HTTP headers passed with the request (only needed for
+        :type headers: list
+        :param headers: HTTP headers passed with the request (only needed for
                         s3 signature checks)
 
-        @rtype: tuple (User, Project)
-        @return: User and project that the request represents.
+        :rtype: tuple (User, Project)
+        :return: User and project that the request represents.
         """
         # TODO(vish): check for valid timestamp
         (access_key, _sep, project_id) = access.partition(':')
@@ -316,7 +331,7 @@ class AuthManager(object):
             LOG.debug(_('user.secret: %s'), user.secret)
             LOG.debug(_('expected_signature: %s'), expected_signature)
             LOG.debug(_('signature: %s'), signature)
-            if signature != expected_signature:
+            if not utils.strcmp_const_time(signature, expected_signature):
                 LOG.audit(_("Invalid signature for user %s"), user.name)
                 raise exception.InvalidSignature(signature=signature,
                                                  user=user)
@@ -328,7 +343,7 @@ class AuthManager(object):
             LOG.debug(_('user.secret: %s'), user.secret)
             LOG.debug(_('expected_signature: %s'), expected_signature)
             LOG.debug(_('signature: %s'), signature)
-            if signature != expected_signature:
+            if not utils.strcmp_const_time(signature, expected_signature):
                 (addr_str, port_str) = utils.parse_server_string(server_string)
                 # If the given server_string contains port num, try without it.
                 if port_str != '':
@@ -337,7 +352,7 @@ class AuthManager(object):
                                                        addr_str, path)
                     LOG.debug(_('host_only_signature: %s'),
                               host_only_signature)
-                    if signature == host_only_signature:
+                    if utils.strcmp_const_time(signature, host_only_signature):
                         return (user, project)
                 LOG.audit(_("Invalid signature for user %s"), user.name)
                 raise exception.InvalidSignature(signature=signature,
@@ -353,11 +368,11 @@ class AuthManager(object):
     def is_superuser(self, user):
         """Checks for superuser status, allowing user to bypass authorization
 
-        @type user: User or uid
-        @param user: User to check.
+        :type user: User or uid
+        :param user: User to check.
 
-        @rtype: bool
-        @return: True for superuser.
+        :rtype: bool
+        :return: True for superuser.
         """
         if not isinstance(user, User):
             user = self.get_user(user)
@@ -371,11 +386,11 @@ class AuthManager(object):
     def is_admin(self, user):
         """Checks for admin status, allowing user to access all projects
 
-        @type user: User or uid
-        @param user: User to check.
+        :type user: User or uid
+        :param user: User to check.
 
-        @rtype: bool
-        @return: True for admin.
+        :rtype: bool
+        :return: True for admin.
         """
         if not isinstance(user, User):
             user = self.get_user(user)
@@ -389,7 +404,7 @@ class AuthManager(object):
         key_parts = ['rolecache', User.safe_id(user), str(role)]
         if project:
             key_parts.append(Project.safe_id(project))
-        return '-'.join(key_parts)
+        return utils.utf8('-'.join(key_parts))
 
     def _clear_mc_key(self, user, role, project=None):
         # NOTE(anthony): it would be better to delete the key
@@ -417,17 +432,17 @@ class AuthManager(object):
         see if the user is the project_manager of the specified project. It
         is the same as calling is_project_manager(user, project).
 
-        @type user: User or uid
-        @param user: User to check.
+        :type user: User or uid
+        :param user: User to check.
 
-        @type role: str
-        @param role: Role to check.
+        :type role: str
+        :param role: Role to check.
 
-        @type project: Project or project_id
-        @param project: Project in which to look for local role.
+        :type project: Project or project_id
+        :param project: Project in which to look for local role.
 
-        @rtype: bool
-        @return: True if the user has the role.
+        :rtype: bool
+        :return: True if the user has the role.
         """
         if role == 'projectmanager':
             if not project:
@@ -456,14 +471,14 @@ class AuthManager(object):
 
         The 'projectmanager' role is special and can't be added or removed.
 
-        @type user: User or uid
-        @param user: User to which to add role.
+        :type user: User or uid
+        :param user: User to which to add role.
 
-        @type role: str
-        @param role: Role to add.
+        :type role: str
+        :param role: Role to add.
 
-        @type project: Project or project_id
-        @param project: Project in which to add local role.
+        :type project: Project or project_id
+        :param project: Project in which to add local role.
         """
         if role not in FLAGS.allowed_roles:
             raise exception.UserRoleNotFound(role_id=role)
@@ -489,14 +504,14 @@ class AuthManager(object):
 
         The 'projectmanager' role is special and can't be added or removed.
 
-        @type user: User or uid
-        @param user: User from which to remove role.
+        :type user: User or uid
+        :param user: User from which to remove role.
 
-        @type role: str
-        @param role: Role to remove.
+        :type role: str
+        :param role: Role to remove.
 
-        @type project: Project or project_id
-        @param project: Project in which to remove local role.
+        :type project: Project or project_id
+        :param project: Project in which to remove local role.
         """
         uid = User.safe_id(user)
         pid = Project.safe_id(project)
@@ -551,23 +566,23 @@ class AuthManager(object):
                        member_users=None):
         """Create a project
 
-        @type name: str
-        @param name: Name of the project to create. The name will also be
+        :type name: str
+        :param name: Name of the project to create. The name will also be
         used as the project id.
 
-        @type manager_user: User or uid
-        @param manager_user: This user will be the project manager.
+        :type manager_user: User or uid
+        :param manager_user: This user will be the project manager.
 
-        @type description: str
-        @param project: Description of the project. If no description is
+        :type description: str
+        :param project: Description of the project. If no description is
         specified, the name of the project will be used.
 
-        @type member_users: list of User or uid
-        @param: Initial project members. The project manager will always be
+        :type member_users: list of User or uid
+        :param: Initial project members. The project manager will always be
         added as a member, even if he isn't specified in this list.
 
-        @rtype: Project
-        @return: The new project.
+        :rtype: Project
+        :return: The new project.
         """
         if member_users:
             member_users = [User.safe_id(u) for u in member_users]
@@ -585,14 +600,14 @@ class AuthManager(object):
     def modify_project(self, project, manager_user=None, description=None):
         """Modify a project
 
-        @type name: Project or project_id
-        @param project: The project to modify.
+        :type name: Project or project_id
+        :param project: The project to modify.
 
-        @type manager_user: User or uid
-        @param manager_user: This user will be the new project manager.
+        :type manager_user: User or uid
+        :param manager_user: This user will be the new project manager.
 
-        @type description: str
-        @param project: This will be the new description of the project.
+        :type description: str
+        :param project: This will be the new description of the project.
 
         """
         LOG.audit(_("modifying project %s"), Project.safe_id(project))
@@ -636,12 +651,12 @@ class AuthManager(object):
     def get_project_vpn_data(project):
         """Gets vpn ip and port for project
 
-        @type project: Project or project_id
-        @param project: Project from which to get associated vpn data
+        :type project: Project or project_id
+        :param project: Project from which to get associated vpn data
 
-        @rvalue: tuple of (str, str)
-        @return: A tuple containing (ip, port) or None, None if vpn has
-        not been allocated for user.
+        :rvalue: tuple of (str, str)
+        :return: A tuple containing (ip, port) or None, None if vpn has
+                 not been allocated for user.
         """
 
         networks = db.project_get_networks(context.get_admin_context(),
@@ -687,24 +702,24 @@ class AuthManager(object):
     def create_user(self, name, access=None, secret=None, admin=False):
         """Creates a user
 
-        @type name: str
-        @param name: Name of the user to create.
+        :type name: str
+        :param name: Name of the user to create.
 
-        @type access: str
-        @param access: Access Key (defaults to a random uuid)
+        :type access: str
+        :param access: Access Key (defaults to a random uuid)
 
-        @type secret: str
-        @param secret: Secret Key (defaults to a random uuid)
+        :type secret: str
+        :param secret: Secret Key (defaults to a random uuid)
 
-        @type admin: bool
-        @param admin: Whether to set the admin flag. The admin flag gives
-        superuser status regardless of roles specifed for the user.
+        :type admin: bool
+        :param admin: Whether to set the admin flag. The admin flag gives
+        superuser status regardless of roles specified for the user.
 
-        @type create_project: bool
-        @param: Whether to create a project for the user with the same name.
+        :type create_project: bool
+        :param: Whether to create a project for the user with the same name.
 
-        @rtype: User
-        @return: The new user.
+        :rtype: User
+        :return: The new user.
         """
         if access is None:
             access = str(uuid.uuid4())
@@ -753,45 +768,44 @@ class AuthManager(object):
         pid = Project.safe_id(project)
         private_key, signed_cert = crypto.generate_x509_cert(user.id, pid)
 
-        tmpdir = tempfile.mkdtemp()
-        zf = os.path.join(tmpdir, "temp.zip")
-        zippy = zipfile.ZipFile(zf, 'w')
-        if use_dmz and FLAGS.region_list:
-            regions = {}
-            for item in FLAGS.region_list:
-                region, _sep, region_host = item.partition("=")
-                regions[region] = region_host
-        else:
-            regions = {'nova': FLAGS.ec2_host}
-        for region, host in regions.iteritems():
-            rc = self.__generate_rc(user,
-                                    pid,
-                                    use_dmz,
-                                    host)
-            zippy.writestr(FLAGS.credential_rc_file % region, rc)
+        with utils.tempdir() as tmpdir:
+            zf = os.path.join(tmpdir, "temp.zip")
+            zippy = zipfile.ZipFile(zf, 'w')
+            if use_dmz and FLAGS.region_list:
+                regions = {}
+                for item in FLAGS.region_list:
+                    region, _sep, region_host = item.partition("=")
+                    regions[region] = region_host
+            else:
+                regions = {'nova': FLAGS.ec2_host}
+            for region, host in regions.iteritems():
+                rc = self.__generate_rc(user,
+                                        pid,
+                                        use_dmz,
+                                        host)
+                zippy.writestr(FLAGS.credential_rc_file % region, rc)
 
-        zippy.writestr(FLAGS.credential_key_file, private_key)
-        zippy.writestr(FLAGS.credential_cert_file, signed_cert)
+            zippy.writestr(FLAGS.credential_key_file, private_key)
+            zippy.writestr(FLAGS.credential_cert_file, signed_cert)
 
-        (vpn_ip, vpn_port) = self.get_project_vpn_data(project)
-        if vpn_ip:
-            configfile = open(FLAGS.vpn_client_template, "r")
-            s = string.Template(configfile.read())
-            configfile.close()
-            config = s.substitute(keyfile=FLAGS.credential_key_file,
-                                  certfile=FLAGS.credential_cert_file,
-                                  ip=vpn_ip,
-                                  port=vpn_port)
-            zippy.writestr(FLAGS.credential_vpn_file, config)
-        else:
-            LOG.warn(_("No vpn data for project %s"), pid)
+            (vpn_ip, vpn_port) = self.get_project_vpn_data(project)
+            if vpn_ip:
+                configfile = open(FLAGS.vpn_client_template, "r")
+                s = string.Template(configfile.read())
+                configfile.close()
+                config = s.substitute(keyfile=FLAGS.credential_key_file,
+                                      certfile=FLAGS.credential_cert_file,
+                                      ip=vpn_ip,
+                                      port=vpn_port)
+                zippy.writestr(FLAGS.credential_vpn_file, config)
+            else:
+                LOG.warn(_("No vpn data for project %s"), pid)
 
-        zippy.writestr(FLAGS.ca_file, crypto.fetch_ca(pid))
-        zippy.close()
-        with open(zf, 'rb') as f:
-            read_buffer = f.read()
+            zippy.writestr(FLAGS.ca_file, crypto.fetch_ca(pid))
+            zippy.close()
+            with open(zf, 'rb') as f:
+                read_buffer = f.read()
 
-        shutil.rmtree(tmpdir)
         return read_buffer
 
     def get_environment_rc(self, user, project=None, use_dmz=True):
@@ -819,7 +833,7 @@ class AuthManager(object):
         rc = open(FLAGS.credentials_template).read()
         # NOTE(vish): Deprecated auth uses an access key, no auth uses a
         #             the user_id in place of it.
-        if FLAGS.use_deprecated_auth:
+        if FLAGS.auth_strategy == 'deprecated':
             access = user.access
         else:
             access = user.id
@@ -833,7 +847,7 @@ class AuthManager(object):
                    's3': 'http://%s:%s' % (s3_host, FLAGS.s3_port),
                    'os': '%s://%s:%s%s' % (FLAGS.osapi_scheme,
                                             ec2_host,
-                                            FLAGS.osapi_port,
+                                            FLAGS.osapi_compute_listen_port,
                                             FLAGS.osapi_path),
                    'user': user.name,
                    'nova': FLAGS.ca_file,
