@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright (c) 2010 Openstack, LLC.
+# Copyright (c) 2010 OpenStack, LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,44 +15,47 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-XVP (Xenserver VNC Proxy) driver.
-"""
+"""XVP (Xenserver VNC Proxy) driver."""
 
-import fcntl
 import os
 import signal
-import subprocess
 
-from Cheetah.Template import Template
+from Cheetah import Template
 
 from nova import context
 from nova import db
 from nova import exception
 from nova import flags
 from nova import log as logging
+from nova.openstack.common import cfg
 from nova import utils
 
-flags.DEFINE_string('console_xvp_conf_template',
-                    utils.abspath('console/xvp.conf.template'),
-                    'XVP conf template')
-flags.DEFINE_string('console_xvp_conf',
-                    '/etc/xvp.conf',
-                    'generated XVP conf file')
-flags.DEFINE_string('console_xvp_pid',
-                    '/var/run/xvp.pid',
-                    'XVP master process pid file')
-flags.DEFINE_string('console_xvp_log',
-                    '/var/log/xvp.log',
-                    'XVP log file')
-flags.DEFINE_integer('console_xvp_multiplex_port',
-                     5900,
-                     "port for XVP to multiplex VNC connections on")
+
+xvp_opts = [
+    cfg.StrOpt('console_xvp_conf_template',
+               default='$pybasedir/nova/console/xvp.conf.template',
+               help='XVP conf template'),
+    cfg.StrOpt('console_xvp_conf',
+               default='/etc/xvp.conf',
+               help='generated XVP conf file'),
+    cfg.StrOpt('console_xvp_pid',
+               default='/var/run/xvp.pid',
+               help='XVP master process pid file'),
+    cfg.StrOpt('console_xvp_log',
+               default='/var/log/xvp.log',
+               help='XVP log file'),
+    cfg.IntOpt('console_xvp_multiplex_port',
+               default=5900,
+               help='port for XVP to multiplex VNC connections on'),
+    ]
+
 FLAGS = flags.FLAGS
+FLAGS.register_opts(xvp_opts)
+LOG = logging.getLogger(__name__)
 
 
 class XVPConsoleProxy(object):
-    """Sets up XVP config, and manages xvp daemon"""
+    """Sets up XVP config, and manages XVP daemon."""
 
     def __init__(self):
         self.xvpconf_template = open(FLAGS.console_xvp_conf_template).read()
@@ -61,60 +64,61 @@ class XVPConsoleProxy(object):
 
     @property
     def console_type(self):
-        return "vnc+xvp"
+        return 'vnc+xvp'
 
     def get_port(self, context):
-        """get available port for consoles that need one"""
+        """Get available port for consoles that need one."""
         #TODO(mdragon): implement port selection for non multiplex ports,
         #               we are not using that, but someone else may want
         #               it.
         return FLAGS.console_xvp_multiplex_port
 
     def setup_console(self, context, console):
-        """Sets up actual proxies"""
+        """Sets up actual proxies."""
         self._rebuild_xvp_conf(context.elevated())
 
     def teardown_console(self, context, console):
-        """Tears down actual proxies"""
+        """Tears down actual proxies."""
         self._rebuild_xvp_conf(context.elevated())
 
     def init_host(self):
-        """Start up any config'ed consoles on start"""
+        """Start up any config'ed consoles on start."""
         ctxt = context.get_admin_context()
         self._rebuild_xvp_conf(ctxt)
 
     def fix_pool_password(self, password):
-        """Trim password to length, and encode"""
+        """Trim password to length, and encode."""
         return self._xvp_encrypt(password, is_pool_password=True)
 
     def fix_console_password(self, password):
-        """Trim password to length, and encode"""
+        """Trim password to length, and encode."""
         return self._xvp_encrypt(password)
 
     def _rebuild_xvp_conf(self, context):
-        logging.debug(_("Rebuilding xvp conf"))
+        LOG.debug(_('Rebuilding xvp conf'))
         pools = [pool for pool in
                  db.console_pool_get_all_by_host_type(context, self.host,
                                                        self.console_type)
                   if pool['consoles']]
         if not pools:
-            logging.debug("No console pools!")
+            LOG.debug('No console pools!')
             self._xvp_stop()
             return
         conf_data = {'multiplex_port': FLAGS.console_xvp_multiplex_port,
                      'pools': pools,
                      'pass_encode': self.fix_console_password}
-        config = str(Template(self.xvpconf_template, searchList=[conf_data]))
+        config = str(Template.Template(self.xvpconf_template,
+                                       searchList=[conf_data]))
         self._write_conf(config)
         self._xvp_restart()
 
     def _write_conf(self, config):
-        logging.debug(_('Re-wrote %s') % FLAGS.console_xvp_conf)
+        LOG.debug(_('Re-wrote %s') % FLAGS.console_xvp_conf)
         with open(FLAGS.console_xvp_conf, 'w') as cfile:
             cfile.write(config)
 
     def _xvp_stop(self):
-        logging.debug(_("Stopping xvp"))
+        LOG.debug(_('Stopping xvp'))
         pid = self._xvp_pid()
         if not pid:
             return
@@ -127,19 +131,19 @@ class XVPConsoleProxy(object):
     def _xvp_start(self):
         if self._xvp_check_running():
             return
-        logging.debug(_("Starting xvp"))
+        LOG.debug(_('Starting xvp'))
         try:
             utils.execute('xvp',
                           '-p', FLAGS.console_xvp_pid,
                           '-c', FLAGS.console_xvp_conf,
                           '-l', FLAGS.console_xvp_log)
         except exception.ProcessExecutionError, err:
-            logging.error(_("Error starting xvp: %s") % err)
+            LOG.error(_('Error starting xvp: %s') % err)
 
     def _xvp_restart(self):
-        logging.debug(_("Restarting xvp"))
+        LOG.debug(_('Restarting xvp'))
         if not self._xvp_check_running():
-            logging.debug(_("xvp not running..."))
+            LOG.debug(_('xvp not running...'))
             self._xvp_start()
         else:
             pid = self._xvp_pid()
@@ -178,7 +182,9 @@ class XVPConsoleProxy(object):
 
         Note that xvp's obfuscation should not be considered 'real' encryption.
         It simply DES encrypts the passwords with static keys plainly viewable
-        in the xvp source code."""
+        in the xvp source code.
+
+        """
         maxlen = 8
         flag = '-e'
         if is_pool_password:
