@@ -1,5 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
+# Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -20,6 +21,15 @@
 
 The underlying driver is loaded as a :class:`LazyPluggable`.
 
+Functions in this module are imported into the nova.db namespace. Call these
+functions from nova.db namespace, not the nova.db.api namespace.
+
+All functions in this module return objects that implement a dictionary-like
+interface. Currently, many of these objects are sqlalchemy objects that
+implement a dictionary interface. However, a future goal is to have all of
+these objects be simple dictionaries.
+
+
 **Related Flags**
 
 :db_backend:  string to lookup in the list of LazyPluggable backends.
@@ -35,39 +45,42 @@ The underlying driver is loaded as a :class:`LazyPluggable`.
 
 from nova import exception
 from nova import flags
+from nova.openstack.common import cfg
 from nova import utils
 
 
+db_opts = [
+    cfg.StrOpt('db_backend',
+               default='sqlalchemy',
+               help='The backend to use for db'),
+    cfg.BoolOpt('enable_new_services',
+                default=True,
+                help='Services to be added to the available pool on create'),
+    cfg.StrOpt('instance_name_template',
+               default='instance-%08x',
+               help='Template string to be used to generate instance names'),
+    cfg.StrOpt('volume_name_template',
+               default='volume-%08x',
+               help='Template string to be used to generate instance names'),
+    cfg.StrOpt('snapshot_name_template',
+               default='snapshot-%08x',
+               help='Template string to be used to generate snapshot names'),
+    ]
+
 FLAGS = flags.FLAGS
-flags.DEFINE_string('db_backend', 'sqlalchemy',
-                    'The backend to use for db')
-flags.DEFINE_boolean('enable_new_services', True,
-                     'Services to be added to the available pool on create')
-flags.DEFINE_string('instance_name_template', 'instance-%08x',
-                    'Template string to be used to generate instance names')
-flags.DEFINE_string('volume_name_template', 'volume-%08x',
-                    'Template string to be used to generate instance names')
-flags.DEFINE_string('snapshot_name_template', 'snapshot-%08x',
-                    'Template string to be used to generate snapshot names')
-flags.DEFINE_string('vsa_name_template', 'vsa-%08x',
-                    'Template string to be used to generate VSA names')
+FLAGS.register_opts(db_opts)
 
-IMPL = utils.LazyPluggable(FLAGS['db_backend'],
+IMPL = utils.LazyPluggable('db_backend',
                            sqlalchemy='nova.db.sqlalchemy.api')
 
 
-class NoMoreBlades(exception.Error):
-    """No more available blades."""
-    pass
-
-
 class NoMoreNetworks(exception.Error):
     """No more available networks."""
     pass
 
 
 class NoMoreTargets(exception.Error):
-    """No more available blades"""
+    """No more available targets"""
     pass
 
 
@@ -118,15 +131,6 @@ def service_get_all_compute_sorted(context):
     return IMPL.service_get_all_compute_sorted(context)
 
 
-def service_get_all_network_sorted(context):
-    """Get all network services sorted by network count.
-
-    :returns: a list of (Service, network_count) tuples.
-
-    """
-    return IMPL.service_get_all_network_sorted(context)
-
-
 def service_get_all_volume_sorted(context):
     """Get all volume services sorted by volume count.
 
@@ -158,25 +162,44 @@ def service_update(context, service_id, values):
 ###################
 
 
-def compute_node_get(context, compute_id, session=None):
+def compute_node_get(context, compute_id):
     """Get an computeNode or raise if it does not exist."""
     return IMPL.compute_node_get(context, compute_id)
 
 
+def compute_node_get_all(context):
+    """Get all computeNodes."""
+    return IMPL.compute_node_get_all(context)
+
+
 def compute_node_create(context, values):
     """Create a computeNode from the values dictionary."""
     return IMPL.compute_node_create(context, values)
 
 
-def compute_node_update(context, compute_id, values):
+def compute_node_update(context, compute_id, values, auto_adjust=True):
     """Set the given properties on an computeNode and update it.
 
     Raises NotFound if computeNode does not exist.
-
     """
+    return IMPL.compute_node_update(context, compute_id, values, auto_adjust)
 
-    return IMPL.compute_node_update(context, compute_id, values)
 
+def compute_node_get_by_host(context, host):
+    return IMPL.compute_node_get_by_host(context, host)
+
+
+def compute_node_utilization_update(context, host, free_ram_mb_delta=0,
+                          free_disk_gb_delta=0, work_delta=0, vm_delta=0):
+    return IMPL.compute_node_utilization_update(context, host,
+                          free_ram_mb_delta, free_disk_gb_delta, work_delta,
+                          vm_delta)
+
+
+def compute_node_utilization_set(context, host, free_ram_mb=None,
+                                 free_disk_gb=None, work=None, vms=None):
+    return IMPL.compute_node_utilization_set(context, host, free_ram_mb,
+                                             free_disk_gb, work, vms)
 
 ###################
 
@@ -186,11 +209,6 @@ def certificate_create(context, values):
     return IMPL.certificate_create(context, values)
 
 
-def certificate_destroy(context, certificate_id):
-    """Destroy the certificate or raise if it does not exist."""
-    return IMPL.certificate_destroy(context, certificate_id)
-
-
 def certificate_get_all_by_project(context, project_id):
     """Get all certificates for a project."""
     return IMPL.certificate_get_all_by_project(context, project_id)
@@ -208,28 +226,24 @@ def certificate_get_all_by_user_and_project(context, user_id, project_id):
                                                         project_id)
 
 
-def certificate_update(context, certificate_id, values):
-    """Set the given properties on an certificate and update it.
-
-    Raises NotFound if service does not exist.
-
-    """
-    return IMPL.certificate_update(context, certificate_id, values)
-
-
 ###################
 
 def floating_ip_get(context, id):
     return IMPL.floating_ip_get(context, id)
 
 
-def floating_ip_allocate_address(context, project_id):
-    """Allocate free floating ip and return the address.
+def floating_ip_get_pools(context):
+    """Returns a list of floating ip pools"""
+    return IMPL.floating_ip_get_pools(context)
+
+
+def floating_ip_allocate_address(context, project_id, pool):
+    """Allocate free floating ip from specified pool and return the address.
 
     Raises if one is not available.
 
     """
-    return IMPL.floating_ip_allocate_address(context, project_id)
+    return IMPL.floating_ip_allocate_address(context, project_id, pool)
 
 
 def floating_ip_create(context, values):
@@ -290,6 +304,16 @@ def floating_ip_get_by_address(context, address):
     return IMPL.floating_ip_get_by_address(context, address)
 
 
+def floating_ip_get_by_fixed_address(context, fixed_address):
+    """Get a floating ips by fixed address"""
+    return IMPL.floating_ip_get_by_fixed_address(context, fixed_address)
+
+
+def floating_ip_get_by_fixed_ip_id(context, fixed_ip_id):
+    """Get a floating ips by fixed address"""
+    return IMPL.floating_ip_get_by_fixed_ip_id(context, fixed_ip_id)
+
+
 def floating_ip_update(context, address, values):
     """Update a floating ip by address or raise if it doesn't exist."""
     return IMPL.floating_ip_update(context, address, values)
@@ -299,6 +323,32 @@ def floating_ip_set_auto_assigned(context, address):
     """Set auto_assigned flag to floating ip"""
     return IMPL.floating_ip_set_auto_assigned(context, address)
 
+
+def dnsdomain_list(context):
+    """Get a list of all zones in our database, public and private."""
+    return IMPL.dnsdomain_list(context)
+
+
+def dnsdomain_register_for_zone(context, fqdomain, zone):
+    """Associated a DNS domain with an availability zone"""
+    return IMPL.dnsdomain_register_for_zone(context, fqdomain, zone)
+
+
+def dnsdomain_register_for_project(context, fqdomain, project):
+    """Associated a DNS domain with a project id"""
+    return IMPL.dnsdomain_register_for_project(context, fqdomain, project)
+
+
+def dnsdomain_unregister(context, fqdomain):
+    """Purge associations for the specified DNS zone"""
+    return IMPL.dnsdomain_unregister(context, fqdomain)
+
+
+def dnsdomain_get(context, fqdomain):
+    """Get the db record for the specified domain."""
+    return IMPL.dnsdomain_get(context, fqdomain)
+
+
 ####################
 
 
@@ -323,6 +373,11 @@ def migration_get_by_instance_and_status(context, instance_uuid, status):
             status)
 
 
+def migration_get_all_unconfirmed(context, confirm_window):
+    """Finds all unconfirmed migrations within the confirmation window."""
+    return IMPL.migration_get_all_unconfirmed(context, confirm_window)
+
+
 ####################
 
 
@@ -367,16 +422,16 @@ def fixed_ip_disassociate_all_by_timeout(context, host, time):
     return IMPL.fixed_ip_disassociate_all_by_timeout(context, host, time)
 
 
+def fixed_ip_get(context, id):
+    """Get fixed ip by id or raise if it does not exist."""
+    return IMPL.fixed_ip_get(context, id)
+
+
 def fixed_ip_get_all(context):
     """Get all defined fixed ips."""
     return IMPL.fixed_ip_get_all(context)
 
 
-def fixed_ip_get_all_by_instance_host(context, host):
-    """Get all allocated fixed ips filtered by instance host."""
-    return IMPL.fixed_ip_get_all_by_instance_host(context, host)
-
-
 def fixed_ip_get_by_address(context, address):
     """Get a fixed ip by address or raise if it does not exist."""
     return IMPL.fixed_ip_get_by_address(context, address)
@@ -392,9 +447,9 @@ def fixed_ip_get_by_network_host(context, network_id, host):
     return IMPL.fixed_ip_get_by_network_host(context, network_id, host)
 
 
-def fixed_ip_get_by_virtual_interface(context, vif_id):
+def fixed_ips_by_virtual_interface(context, vif_id):
     """Get fixed ips by virtual interface or raise if none exist."""
-    return IMPL.fixed_ip_get_by_virtual_interface(context, vif_id)
+    return IMPL.fixed_ips_by_virtual_interface(context, vif_id)
 
 
 def fixed_ip_get_network(context, address):
@@ -414,11 +469,6 @@ def virtual_interface_create(context, values):
     return IMPL.virtual_interface_create(context, values)
 
 
-def virtual_interface_update(context, vif_id, values):
-    """Update a virtual interface record in the database."""
-    return IMPL.virtual_interface_update(context, vif_id, values)
-
-
 def virtual_interface_get(context, vif_id):
     """Gets a virtual interface from the table,"""
     return IMPL.virtual_interface_get(context, vif_id)
@@ -434,11 +484,6 @@ def virtual_interface_get_by_uuid(context, vif_uuid):
     return IMPL.virtual_interface_get_by_uuid(context, vif_uuid)
 
 
-def virtual_interface_get_by_fixed_ip(context, fixed_ip_id):
-    """Gets the virtual interface fixed_ip is associated with."""
-    return IMPL.virtual_interface_get_by_fixed_ip(context, fixed_ip_id)
-
-
 def virtual_interface_get_by_instance(context, instance_id):
     """Gets all virtual_interfaces for instance."""
     return IMPL.virtual_interface_get_by_instance(context, instance_id)
@@ -452,11 +497,6 @@ def virtual_interface_get_by_instance_and_network(context, instance_id,
                                                               network_id)
 
 
-def virtual_interface_get_by_network(context, network_id):
-    """Gets all virtual interfaces on network."""
-    return IMPL.virtual_interface_get_by_network(context, network_id)
-
-
 def virtual_interface_delete(context, vif_id):
     """Delete virtual interface record from the database."""
     return IMPL.virtual_interface_delete(context, vif_id)
@@ -467,6 +507,11 @@ def virtual_interface_delete_by_instance(context, instance_id):
     return IMPL.virtual_interface_delete_by_instance(context, instance_id)
 
 
+def virtual_interface_get_all(context):
+    """Gets all virtual interfaces from the table"""
+    return IMPL.virtual_interface_get_all(context)
+
+
 ####################
 
 
@@ -485,11 +530,6 @@ def instance_destroy(context, instance_id):
     return IMPL.instance_destroy(context, instance_id)
 
 
-def instance_stop(context, instance_id):
-    """Stop the instance or raise if it does not exist."""
-    return IMPL.instance_stop(context, instance_id)
-
-
 def instance_get_by_uuid(context, uuid):
     """Get an instance or raise if it does not exist."""
     return IMPL.instance_get_by_uuid(context, uuid)
@@ -505,9 +545,11 @@ def instance_get_all(context):
     return IMPL.instance_get_all(context)
 
 
-def instance_get_all_by_filters(context, filters):
+def instance_get_all_by_filters(context, filters, sort_key='created_at',
+                                sort_dir='desc'):
     """Get all instances that match all filters."""
-    return IMPL.instance_get_all_by_filters(context, filters)
+    return IMPL.instance_get_all_by_filters(context, filters, sort_key,
+                                            sort_dir)
 
 
 def instance_get_active_by_window(context, begin, end=None, project_id=None):
@@ -526,11 +568,6 @@ def instance_get_active_by_window_joined(context, begin, end=None,
                                               project_id)
 
 
-def instance_get_all_by_user(context, user_id):
-    """Get all instances."""
-    return IMPL.instance_get_all_by_user(context, user_id)
-
-
 def instance_get_all_by_project(context, project_id):
     """Get all instance belonging to a project."""
     return IMPL.instance_get_all_by_project(context, project_id)
@@ -546,38 +583,23 @@ def instance_get_all_by_reservation(context, reservation_id):
     return IMPL.instance_get_all_by_reservation(context, reservation_id)
 
 
-def instance_get_by_fixed_ip(context, address):
-    """Get an instance for a fixed ip by address."""
-    return IMPL.instance_get_by_fixed_ip(context, address)
-
-
-def instance_get_by_fixed_ipv6(context, address):
-    """Get an instance for a fixed ip by IPv6 address."""
-    return IMPL.instance_get_by_fixed_ipv6(context, address)
-
-
-def instance_get_fixed_addresses(context, instance_id):
-    """Get the fixed ip address of an instance."""
-    return IMPL.instance_get_fixed_addresses(context, instance_id)
-
-
-def instance_get_fixed_addresses_v6(context, instance_id):
-    return IMPL.instance_get_fixed_addresses_v6(context, instance_id)
-
-
 def instance_get_floating_address(context, instance_id):
     """Get the first floating ip address of an instance."""
     return IMPL.instance_get_floating_address(context, instance_id)
 
 
-def instance_get_project_vpn(context, project_id):
-    """Get a vpn instance by project or return None."""
-    return IMPL.instance_get_project_vpn(context, project_id)
+def instance_get_all_hung_in_rebooting(context, reboot_window):
+    """Get all instances stuck in a rebooting state."""
+    return IMPL.instance_get_all_hung_in_rebooting(context, reboot_window)
 
 
-def instance_set_state(context, instance_id, state, description=None):
-    """Set the state of an instance."""
-    return IMPL.instance_set_state(context, instance_id, state, description)
+def instance_test_and_set(context, instance_id, attr, ok_states,
+                          new_state):
+    """Atomically check if an instance is in a valid state, and if it is, set
+    the instance into a new state.
+    """
+    return IMPL.instance_test_and_set(
+            context, instance_id, attr, ok_states, new_state)
 
 
 def instance_update(context, instance_id, values):
@@ -606,9 +628,51 @@ def instance_action_create(context, values):
     return IMPL.instance_action_create(context, values)
 
 
-def instance_get_actions(context, instance_id):
-    """Get instance actions by instance id."""
-    return IMPL.instance_get_actions(context, instance_id)
+def instance_get_actions(context, instance_uuid):
+    """Get instance actions by instance uuid."""
+    return IMPL.instance_get_actions(context, instance_uuid)
+
+
+def instance_get_id_to_uuid_mapping(context, ids):
+    """Return a dictionary containing 'ID: UUID' given the ids"""
+    return IMPL.instance_get_id_to_uuid_mapping(context, ids)
+
+
+###################
+
+
+def instance_info_cache_create(context, values):
+    """Create a new instance cache record in the table.
+
+    :param context: = request context object
+    :param values: = dict containing column values
+    """
+    return IMPL.instance_info_cache_create(context, values)
+
+
+def instance_info_cache_get(context, instance_uuid):
+    """Gets an instance info cache from the table.
+
+    :param instance_uuid: = uuid of the info cache's instance
+    """
+    return IMPL.instance_info_cache_get(context, instance_uuid)
+
+
+def instance_info_cache_update(context, instance_uuid, values):
+    """Update an instance info cache record in the table.
+
+    :param instance_uuid: = uuid of info cache's instance
+    :param values: = dict containing column values to update
+    """
+    return IMPL.instance_info_cache_update(context, instance_uuid, values)
+
+
+def instance_info_cache_delete(context, instance_uuid):
+    """Deletes an existing instance_info_cache record
+
+    :param instance_uuid: = uuid of the instance tied to the cache record
+    """
+    return IMPL.instance_info_cache_delete(context, instance_uuid)
 
 
 ###################
@@ -652,16 +716,6 @@ def network_count(context):
     return IMPL.network_count(context)
 
 
-def network_count_allocated_ips(context, network_id):
-    """Return the number of allocated non-reserved ips in the network."""
-    return IMPL.network_count_allocated_ips(context, network_id)
-
-
-def network_count_available_ips(context, network_id):
-    """Return the number of available ips in the network."""
-    return IMPL.network_count_available_ips(context, network_id)
-
-
 def network_count_reserved_ips(context, network_id):
     """Return the number of reserved ips in the network."""
     return IMPL.network_count_reserved_ips(context, network_id)
@@ -696,11 +750,6 @@ def network_disassociate(context, network_id):
     return IMPL.network_disassociate(context, network_id)
 
 
-def network_disassociate_all(context):
-    """Disassociate all networks from projects."""
-    return IMPL.network_disassociate_all(context)
-
-
 def network_get(context, network_id):
     """Get an network or raise if it does not exist."""
     return IMPL.network_get(context, network_id)
@@ -719,9 +768,9 @@ def network_get_all_by_uuids(context, network_uuids, project_id=None):
 # pylint: disable=C0103
 
 
-def network_get_associated_fixed_ips(context, network_id):
+def network_get_associated_fixed_ips(context, network_id, host=None):
     """Get all network's ips that have been associated."""
-    return IMPL.network_get_associated_fixed_ips(context, network_id)
+    return IMPL.network_get_associated_fixed_ips(context, network_id, host)
 
 
 def network_get_by_bridge(context, bridge):
@@ -759,11 +808,6 @@ def network_get_index(context, network_id):
     return IMPL.network_get_index(context, network_id)
 
 
-def network_get_vpn_ip(context, network_id):
-    """Get non-conflicting index for network."""
-    return IMPL.network_get_vpn_ip(context, network_id)
-
-
 def network_set_cidr(context, network_id, cidr):
     """Set the Classless Inner Domain Routing for the network."""
     return IMPL.network_set_cidr(context, network_id, cidr)
@@ -794,25 +838,6 @@ def queue_get_for(context, topic, physical_node_id):
 ###################
 
 
-def export_device_count(context):
-    """Return count of export devices."""
-    return IMPL.export_device_count(context)
-
-
-def export_device_create_safe(context, values):
-    """Create an export_device from the values dictionary.
-
-    The device is not returned. If the create violates the unique
-    constraints because the shelf_id and blade_id already exist,
-    no exception is raised.
-
-    """
-    return IMPL.export_device_create_safe(context, values)
-
-
-###################
-
-
 def iscsi_target_count_by_host(context, host):
     """Return count of export devices."""
     return IMPL.iscsi_target_count_by_host(context, host)
@@ -888,11 +913,6 @@ def quota_destroy_all_by_project(context, project_id):
 ###################
 
 
-def volume_allocate_shelf_and_blade(context, volume_id):
-    """Atomically allocate a free shelf and blade from the pool."""
-    return IMPL.volume_allocate_shelf_and_blade(context, volume_id)
-
-
 def volume_allocate_iscsi_target(context, volume_id, host):
     """Atomically allocate a free iscsi_target from the pool."""
     return IMPL.volume_allocate_iscsi_target(context, volume_id, host)
@@ -958,11 +978,6 @@ def volume_get_instance(context, volume_id):
     return IMPL.volume_get_instance(context, volume_id)
 
 
-def volume_get_shelf_and_blade(context, volume_id):
-    """Get the shelf and blade allocated to the volume."""
-    return IMPL.volume_get_shelf_and_blade(context, volume_id)
-
-
 def volume_get_iscsi_target_num(context, volume_id):
     """Get the target num (tid) allocated to the volume."""
     return IMPL.volume_get_iscsi_target_num(context, volume_id)
@@ -1005,6 +1020,11 @@ def snapshot_get_all_by_project(context, project_id):
     return IMPL.snapshot_get_all_by_project(context, project_id)
 
 
+def snapshot_get_all_for_volume(context, volume_id):
+    """Get all snapshots for a volume."""
+    return IMPL.snapshot_get_all_for_volume(context, volume_id)
+
+
 def snapshot_update(context, snapshot_id, values):
     """Set the given properties on an snapshot and update it.
 
@@ -1083,6 +1103,11 @@ def security_group_exists(context, project_id, group_name):
     return IMPL.security_group_exists(context, project_id, group_name)
 
 
+def security_group_in_use(context, group_id):
+    """Indicates if a security group is currently in use."""
+    return IMPL.security_group_in_use(context, group_id)
+
+
 def security_group_create(context, values):
     """Create a new security group."""
     return IMPL.security_group_create(context, values)
@@ -1093,16 +1118,6 @@ def security_group_destroy(context, security_group_id):
     return IMPL.security_group_destroy(context, security_group_id)
 
 
-def security_group_destroy_all(context):
-    """Deletes a security group."""
-    return IMPL.security_group_destroy_all(context)
-
-
-def security_group_count_by_project(context, project_id):
-    """Count number of security groups in a project."""
-    return IMPL.security_group_count_by_project(context, project_id)
-
-
 ####################
 
 
@@ -1134,11 +1149,6 @@ def security_group_rule_get(context, security_group_rule_id):
     return IMPL.security_group_rule_get(context, security_group_rule_id)
 
 
-def security_group_rule_count_by_group(context, security_group_id):
-    """Count rules in a given security group."""
-    return IMPL.security_group_rule_count_by_group(context, security_group_id)
-
-
 ###################
 
 
@@ -1152,11 +1162,6 @@ def provider_fw_rule_get_all(context):
     return IMPL.provider_fw_rule_get_all(context)
 
 
-def provider_fw_rule_get_all_by_cidr(context, cidr):
-    """Get all provider-level firewall rules."""
-    return IMPL.provider_fw_rule_get_all_by_cidr(context, cidr)
-
-
 def provider_fw_rule_destroy(context, rule_id):
     """Delete a provider firewall rule from the database."""
     return IMPL.provider_fw_rule_destroy(context, rule_id)
@@ -1283,10 +1288,6 @@ def project_get_networks(context, project_id, associate=True):
     return IMPL.project_get_networks(context, project_id, associate)
 
 
-def project_get_networks_v6(context, project_id):
-    return IMPL.project_get_networks_v6(context, project_id)
-
-
 ###################
 
 
@@ -1349,9 +1350,10 @@ def instance_type_create(context, values):
     return IMPL.instance_type_create(context, values)
 
 
-def instance_type_get_all(context, inactive=False):
+def instance_type_get_all(context, inactive=False, filters=None):
     """Get all instance types."""
-    return IMPL.instance_type_get_all(context, inactive)
+    return IMPL.instance_type_get_all(
+        context, inactive=inactive, filters=filters)
 
 
 def instance_type_get(context, id):
@@ -1374,41 +1376,32 @@ def instance_type_destroy(context, name):
     return IMPL.instance_type_destroy(context, name)
 
 
-def instance_type_purge(context, name):
-    """Purges (removes) an instance type from DB.
-
-    Use instance_type_destroy for most cases
-
-    """
-    return IMPL.instance_type_purge(context, name)
-
-
 ####################
 
 
-def zone_create(context, values):
-    """Create a new child Zone entry."""
-    return IMPL.zone_create(context, values)
+def cell_create(context, values):
+    """Create a new child Cell entry."""
+    return IMPL.cell_create(context, values)
 
 
-def zone_update(context, zone_id, values):
-    """Update a child Zone entry."""
-    return IMPL.zone_update(context, zone_id, values)
+def cell_update(context, cell_id, values):
+    """Update a child Cell entry."""
+    return IMPL.cell_update(context, cell_id, values)
 
 
-def zone_delete(context, zone_id):
-    """Delete a child Zone."""
-    return IMPL.zone_delete(context, zone_id)
+def cell_delete(context, cell_id):
+    """Delete a child Cell."""
+    return IMPL.cell_delete(context, cell_id)
 
 
-def zone_get(context, zone_id):
-    """Get a specific child Zone."""
-    return IMPL.zone_get(context, zone_id)
+def cell_get(context, cell_id):
+    """Get a specific child Cell."""
+    return IMPL.cell_get(context, cell_id)
 
 
-def zone_get_all(context):
-    """Get all child Zones."""
-    return IMPL.zone_get_all(context)
+def cell_get_all(context):
+    """Get all child Cells."""
+    return IMPL.cell_get_all(context)
 
 
 ####################
@@ -1461,6 +1454,26 @@ def agent_build_update(context, agent_build_id, values):
 ####################
 
 
+def bw_usage_get_by_macs(context, macs, start_period):
+    """Return bw usages for an instance in a given audit period."""
+    return IMPL.bw_usage_get_by_macs(context, macs, start_period)
+
+
+def bw_usage_update(context,
+                    mac,
+                    start_period,
+                    bw_in, bw_out):
+    """Update cached bw usage for an instance and network
+       Creates new record if needed."""
+    return IMPL.bw_usage_update(context,
+                                mac,
+                                start_period,
+                                bw_in, bw_out)
+
+
+####################
+
+
 def instance_type_extra_specs_get(context, instance_type_id):
     """Get all extra specs for an instance type."""
     return IMPL.instance_type_extra_specs_get(context, instance_type_id)
@@ -1525,15 +1538,6 @@ def volume_type_destroy(context, name):
     return IMPL.volume_type_destroy(context, name)
 
 
-def volume_type_purge(context, name):
-    """Purges (removes) a volume type from DB.
-
-    Use volume_type_destroy for most cases
-
-    """
-    return IMPL.volume_type_purge(context, name)
-
-
 ####################
 
 
@@ -1555,34 +1559,185 @@ def volume_type_extra_specs_update_or_create(context, volume_type_id,
                                                     extra_specs)
 
 
+###################
+
+
+def s3_image_get(context, image_id):
+    """Find local s3 image represented by the provided id"""
+    return IMPL.s3_image_get(context, image_id)
+
+
+def s3_image_get_by_uuid(context, image_uuid):
+    """Find local s3 image represented by the provided uuid"""
+    return IMPL.s3_image_get_by_uuid(context, image_uuid)
+
+
+def s3_image_create(context, image_uuid):
+    """Create local s3 image represented by provided uuid"""
+    return IMPL.s3_image_create(context, image_uuid)
+
+
 ####################
 
 
-def vsa_create(context, values):
-    """Creates Virtual Storage Array record."""
-    return IMPL.vsa_create(context, values)
+def sm_backend_conf_create(context, values):
+    """Create a new SM Backend Config entry."""
+    return IMPL.sm_backend_conf_create(context, values)
 
 
-def vsa_update(context, vsa_id, values):
-    """Updates Virtual Storage Array record."""
-    return IMPL.vsa_update(context, vsa_id, values)
+def sm_backend_conf_update(context, sm_backend_conf_id, values):
+    """Update a SM Backend Config entry."""
+    return IMPL.sm_backend_conf_update(context, sm_backend_conf_id, values)
 
 
-def vsa_destroy(context, vsa_id):
-    """Deletes Virtual Storage Array record."""
-    return IMPL.vsa_destroy(context, vsa_id)
+def sm_backend_conf_delete(context, sm_backend_conf_id):
+    """Delete a SM Backend Config."""
+    return IMPL.sm_backend_conf_delete(context, sm_backend_conf_id)
 
 
-def vsa_get(context, vsa_id):
-    """Get Virtual Storage Array record by ID."""
-    return IMPL.vsa_get(context, vsa_id)
+def sm_backend_conf_get(context, sm_backend_conf_id):
+    """Get a specific SM Backend Config."""
+    return IMPL.sm_backend_conf_get(context, sm_backend_conf_id)
 
 
-def vsa_get_all(context):
-    """Get all Virtual Storage Array records."""
-    return IMPL.vsa_get_all(context)
+def sm_backend_conf_get_by_sr(context, sr_uuid):
+    """Get a specific SM Backend Config."""
+    return IMPL.sm_backend_conf_get_by_sr(context, sr_uuid)
 
 
-def vsa_get_all_by_project(context, project_id):
-    """Get all Virtual Storage Array records by project ID."""
-    return IMPL.vsa_get_all_by_project(context, project_id)
+def sm_backend_conf_get_all(context):
+    """Get all SM Backend Configs."""
+    return IMPL.sm_backend_conf_get_all(context)
+
+
+####################
+
+
+def sm_flavor_create(context, values):
+    """Create a new SM Flavor entry."""
+    return IMPL.sm_flavor_create(context, values)
+
+
+def sm_flavor_update(context, sm_flavor_id, values):
+    """Update a SM Flavor entry."""
+    return IMPL.sm_flavor_update(context, values)
+
+
+def sm_flavor_delete(context, sm_flavor_id):
+    """Delete a SM Flavor."""
+    return IMPL.sm_flavor_delete(context, sm_flavor_id)
+
+
+def sm_flavor_get(context, sm_flavor):
+    """Get a specific SM Flavor."""
+    return IMPL.sm_flavor_get(context, sm_flavor)
+
+
+def sm_flavor_get_all(context):
+    """Get all SM Flavors."""
+    return IMPL.sm_flavor_get_all(context)
+
+
+####################
+
+
+def sm_volume_create(context, values):
+    """Create a new child Zone entry."""
+    return IMPL.sm_volume_create(context, values)
+
+
+def sm_volume_update(context, volume_id, values):
+    """Update a child Zone entry."""
+    return IMPL.sm_volume_update(context, values)
+
+
+def sm_volume_delete(context, volume_id):
+    """Delete a child Zone."""
+    return IMPL.sm_volume_delete(context, volume_id)
+
+
+def sm_volume_get(context, volume_id):
+    """Get a specific child Zone."""
+    return IMPL.sm_volume_get(context, volume_id)
+
+
+def sm_volume_get_all(context):
+    """Get all child Zones."""
+    return IMPL.sm_volume_get_all(context)
+
+
+####################
+
+
+def aggregate_create(context, values, metadata=None):
+    """Create a new aggregate with metadata."""
+    return IMPL.aggregate_create(context, values, metadata)
+
+
+def aggregate_get(context, aggregate_id, read_deleted='no'):
+    """Get a specific aggregate by id."""
+    return IMPL.aggregate_get(context, aggregate_id, read_deleted)
+
+
+def aggregate_get_by_host(context, host, read_deleted='no'):
+    """Get a specific aggregate by host"""
+    return IMPL.aggregate_get_by_host(context, host, read_deleted)
+
+
+def aggregate_update(context, aggregate_id, values):
+    """Update the attributes of an aggregates. If values contains a metadata
+    key, it updates the aggregate metadata too."""
+    return IMPL.aggregate_update(context, aggregate_id, values)
+
+
+def aggregate_delete(context, aggregate_id):
+    """Delete an aggregate."""
+    return IMPL.aggregate_delete(context, aggregate_id)
+
+
+def aggregate_get_all(context, read_deleted='yes'):
+    """Get all aggregates."""
+    return IMPL.aggregate_get_all(context, read_deleted)
+
+
+def aggregate_metadata_add(context, aggregate_id, metadata, set_delete=False):
+    """Add/update metadata. If set_delete=True, it adds only."""
+    IMPL.aggregate_metadata_add(context, aggregate_id, metadata, set_delete)
+
+
+def aggregate_metadata_get(context, aggregate_id, read_deleted='no'):
+    """Get metadata for the specified aggregate."""
+    return IMPL.aggregate_metadata_get(context, aggregate_id, read_deleted)
+
+
+def aggregate_metadata_delete(context, aggregate_id, key):
+    """Delete the given metadata key."""
+    IMPL.aggregate_metadata_delete(context, aggregate_id, key)
+
+
+def aggregate_host_add(context, aggregate_id, host):
+    """Add host to the aggregate."""
+    IMPL.aggregate_host_add(context, aggregate_id, host)
+
+
+def aggregate_host_get_all(context, aggregate_id, read_deleted='yes'):
+    """Get hosts for the specified aggregate."""
+    return IMPL.aggregate_host_get_all(context, aggregate_id, read_deleted)
+
+
+def aggregate_host_delete(context, aggregate_id, host):
+    """Delete the given host from the aggregate."""
+    IMPL.aggregate_host_delete(context, aggregate_id, host)
+
+
+####################
+
+
+def instance_fault_create(context, values):
+    """Create a new Instance Fault."""
+    return IMPL.instance_fault_create(context, values)
+
+
+def instance_fault_get_by_instance_uuids(context, instance_uuids):
+    """Get all instance faults for the provided instance_uuids."""
+    return IMPL.instance_fault_get_by_instance_uuids(context, instance_uuids)
