@@ -16,20 +16,24 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Base class for classes that need modular database access.
-"""
+"""Base class for classes that need modular database access."""
 
 from nova import utils
 from nova import flags
+from nova.openstack.common import cfg
+
+
+db_driver_opt = cfg.StrOpt('db_driver',
+                           default='nova.db',
+                           help='driver to use for database access')
 
 FLAGS = flags.FLAGS
-flags.DEFINE_string('db_driver', 'nova.db.api',
-                    'driver to use for database access')
+FLAGS.register_opt(db_driver_opt)
 
 
 class Base(object):
-    """DB driver is injected in the init method"""
+    """DB driver is injected in the init method."""
+
     def __init__(self, db_driver=None):
         if not db_driver:
             db_driver = FLAGS.db_driver
