@@ -19,6 +19,7 @@
 import os
 import sys
 
+from nova import exception
 from nova import flags
 
 import sqlalchemy
@@ -38,9 +39,19 @@ FLAGS = flags.FLAGS
 
 
 def db_sync(version=None):
-    db_version()
+    if version is not None:
+        try:
+            version = int(version)
+        except ValueError:
+            raise exception.Error(_("version should be an integer"))
+
+    current_version = db_version()
     repo_path = _find_migrate_repo()
-    return versioning_api.upgrade(FLAGS.sql_connection, repo_path, version)
+    if version is None or version > current_version:
+        return versioning_api.upgrade(FLAGS.sql_connection, repo_path, version)
+    else:
+        return versioning_api.downgrade(FLAGS.sql_connection, repo_path,
+                                        version)
 
 
 def db_version():
@@ -64,6 +75,7 @@ def db_version():
                           'users', 'user_project_association',
                           'user_project_role_association',
                           'user_role_association',
+                          'virtual_storage_arrays',
                           'volumes', 'volume_metadata',
                           'volume_types', 'volume_type_extra_specs'):
                 assert table in meta.tables
