@@ -24,17 +24,34 @@ SHOULD include dedicated exception logging.
 
 """
 
-from functools import wraps
+import functools
 import sys
 
+import novaclient.exceptions
+import webob.exc
+
 from nova import log as logging
 
-LOG = logging.getLogger('nova.exception')
+LOG = logging.getLogger(__name__)
+
+
+class ConvertedException(webob.exc.WSGIHTTPException):
+    def __init__(self, code=0, title="", explanation=""):
+        self.code = code
+        self.title = title
+        self.explanation = explanation
+        super(ConvertedException, self).__init__()
 
 
 class ProcessExecutionError(IOError):
     def __init__(self, stdout=None, stderr=None, exit_code=None, cmd=None,
                  description=None):
+        self.exit_code = exit_code
+        self.stderr = stderr
+        self.stdout = stdout
+        self.cmd = cmd
+        self.description = description
+
         if description is None:
             description = _('Unexpected error while running command.')
         if exit_code is None:
@@ -46,11 +63,10 @@ class ProcessExecutionError(IOError):
 
 
 class Error(Exception):
-    def __init__(self, message=None):
-        super(Error, self).__init__(message)
+    pass
 
 
-class ApiError(Error):
+class EC2APIError(Error):
     def __init__(self, message='Unknown', code=None):
         self.msg = message
         self.code = code
@@ -58,11 +74,7 @@ class ApiError(Error):
             outstr = '%s: %s' % (code, message)
         else:
             outstr = '%s' % message
-        super(ApiError, self).__init__(outstr)
-
-
-class RebuildRequiresActiveInstance(Error):
-    pass
+        super(EC2APIError, self).__init__(outstr)
 
 
 class DBError(Error):
@@ -76,6 +88,8 @@ def wrap_db_error(f):
     def _wrap(*args, **kwargs):
         try:
             return f(*args, **kwargs)
+        except UnicodeEncodeError:
+            raise InvalidUnicodeParameter()
         except Exception, e:
             LOG.exception(_('DB exception wrapped.'))
             raise DBError(e)
@@ -126,7 +140,7 @@ def wrap_exception(notifier=None, publisher_id=None, event_type=None,
                 # re-raise original exception since it may have been clobbered
                 raise exc_info[0], exc_info[1], exc_info[2]
 
-        return wraps(f)(wrapped)
+        return functools.wraps(f)(wrapped)
     return inner
 
 
@@ -140,17 +154,28 @@ class NovaException(Exception):
     """
     message = _("An unknown exception occurred.")
 
-    def __init__(self, **kwargs):
+    def __init__(self, message=None, **kwargs):
         self.kwargs = kwargs
-        try:
-            self._error_string = self.message % kwargs
 
-        except Exception:
-            # at least get the core message out if something happened
-            self._error_string = self.message
+        if 'code' not in self.kwargs:
+            try:
+                self.kwargs['code'] = self.code
+            except AttributeError:
+                pass
 
-    def __str__(self):
-        return self._error_string
+        if not message:
+            try:
+                message = self.message % kwargs
+
+            except Exception as e:
+                # at least get the core message out if something happened
+                message = self.message
+
+        super(NovaException, self).__init__(message)
+
+
+class DecryptionFailure(NovaException):
+    message = _("Failed to decrypt text")
 
 
 class ImagePaginationFailed(NovaException):
@@ -166,31 +191,54 @@ class VirtualInterfaceMacAddressException(NovaException):
                 "with unique mac address failed")
 
 
+class GlanceConnectionFailed(NovaException):
+    message = _("Connection to glance failed") + ": %(reason)s"
+
+
+class MelangeConnectionFailed(NovaException):
+    message = _("Connection to melange failed") + ": %(reason)s"
+
+
 class NotAuthorized(NovaException):
     message = _("Not authorized.")
-
-    def __init__(self, *args, **kwargs):
-        super(NotAuthorized, self).__init__(**kwargs)
+    code = 403
 
 
 class AdminRequired(NotAuthorized):
     message = _("User does not have admin privileges")
 
 
-class InstanceBusy(NovaException):
-    message = _("Instance %(instance_id)s is busy. (%(task_state)s)")
+class PolicyNotAuthorized(NotAuthorized):
+    message = _("Policy doesn't allow %(action)s to be performed.")
 
 
-class InstanceSnapshotting(InstanceBusy):
-    message = _("Instance %(instance_id)s is currently snapshotting.")
-
-
-class InstanceBackingUp(InstanceBusy):
-    message = _("Instance %(instance_id)s is currently being backed up.")
+class ImageNotAuthorized(NovaException):
+    message = _("Not authorized for image %(image_id)s.")
 
 
 class Invalid(NovaException):
     message = _("Unacceptable parameters.")
+    code = 400
+
+
+class InvalidSnapshot(Invalid):
+    message = _("Invalid snapshot") + ": %(reason)s"
+
+
+class VolumeUnattached(Invalid):
+    message = _("Volume %(volume_id)s is not attached to anything")
+
+
+class InvalidKeypair(Invalid):
+    message = _("Keypair data is invalid")
+
+
+class SfJsonEncodeFailure(NovaException):
+    message = _("Failed to load data into json format")
+
+
+class InvalidRequest(Invalid):
+    message = _("The request is invalid.")
 
 
 class InvalidSignature(Invalid):
@@ -206,7 +254,11 @@ class InvalidInstanceType(Invalid):
 
 
 class InvalidVolumeType(Invalid):
-    message = _("Invalid volume type %(volume_type)s.")
+    message = _("Invalid volume type") + ": %(reason)s"
+
+
+class InvalidVolume(Invalid):
+    message = _("Invalid volume") + ": %(reason)s"
 
 
 class InvalidPortRange(Invalid):
@@ -225,12 +277,35 @@ class InvalidCidr(Invalid):
     message = _("Invalid cidr %(cidr)s.")
 
 
+class InvalidRPCConnectionReuse(Invalid):
+    message = _("Invalid reuse of an RPC connection.")
+
+
+class InvalidUnicodeParameter(Invalid):
+    message = _("Invalid Parameter: "
+                "Unicode is not supported by the current database.")
+
+
 # Cannot be templated as the error syntax varies.
 # msg needs to be constructed when raised.
 class InvalidParameterValue(Invalid):
     message = _("%(err)s")
 
 
+class InvalidAggregateAction(Invalid):
+    message = _("Cannot perform action '%(action)s' on aggregate "
+                "%(aggregate_id)s. Reason: %(reason)s.")
+
+
+class InvalidGroup(Invalid):
+    message = _("Group not valid. Reason: %(reason)s")
+
+
+class InstanceInvalidState(Invalid):
+    message = _("Instance %(instance_uuid)s in %(attr)s %(state)s. Cannot "
+                "%(method)s while the instance is in this state.")
+
+
 class InstanceNotRunning(Invalid):
     message = _("Instance %(instance_id)s is not running.")
 
@@ -255,6 +330,10 @@ class InstanceRebootFailure(Invalid):
     message = _("Failed to reboot instance") + ": %(reason)s"
 
 
+class InstanceTerminationFailure(Invalid):
+    message = _("Failed to terminate instance") + ": %(reason)s"
+
+
 class ServiceUnavailable(Invalid):
     message = _("Service is unavailable at this time.")
 
@@ -272,6 +351,10 @@ class UnableToMigrateToSelf(Invalid):
                 "to current host (%(host)s).")
 
 
+class DestinationHostUnavailable(Invalid):
+    message = _("Destination compute host is unavailable at this time.")
+
+
 class SourceHostUnavailable(Invalid):
     message = _("Original compute host is unavailable at this time.")
 
@@ -294,10 +377,18 @@ class InvalidDevicePath(Invalid):
     message = _("The supplied device path (%(path)s) is invalid.")
 
 
+class DeviceIsBusy(Invalid):
+    message = _("The supplied device (%(device)s) is busy.")
+
+
 class InvalidCPUInfo(Invalid):
     message = _("Unacceptable CPU info") + ": %(reason)s"
 
 
+class InvalidIpAddressError(Invalid):
+    message = _("%(address)s is not a valid IP v4/6 address.")
+
+
 class InvalidVLANTag(Invalid):
     message = _("VLAN tag is not appropriate for the port group "
                 "%(bridge)s. Expected VLAN tag is %(tag)s, "
@@ -316,11 +407,11 @@ class InvalidDiskFormat(Invalid):
 
 
 class ImageUnacceptable(Invalid):
-    message = _("Image %(image_id)s is unacceptable") + ": %(reason)s"
+    message = _("Image %(image_id)s is unacceptable: %(reason)s")
 
 
 class InstanceUnacceptable(Invalid):
-    message = _("Instance %(instance_id)s is unacceptable") + ": %(reason)s"
+    message = _("Instance %(instance_id)s is unacceptable: %(reason)s")
 
 
 class InvalidEc2Id(Invalid):
@@ -329,23 +420,22 @@ class InvalidEc2Id(Invalid):
 
 class NotFound(NovaException):
     message = _("Resource could not be found.")
-
-    def __init__(self, *args, **kwargs):
-        super(NotFound, self).__init__(**kwargs)
+    code = 404
 
 
 class FlagNotSet(NotFound):
     message = _("Required flag %(flag)s not set.")
 
 
-class InstanceNotFound(NotFound):
-    message = _("Instance %(instance_id)s could not be found.")
-
-
 class VolumeNotFound(NotFound):
     message = _("Volume %(volume_id)s could not be found.")
 
 
+class SfAccountNotFound(NotFound):
+    message = _("Unable to locate account %(account_name)s on "
+                "Solidfire device")
+
+
 class VolumeNotFoundForInstance(VolumeNotFound):
     message = _("Volume not found for instance %(instance_id)s.")
 
@@ -381,8 +471,9 @@ class VolumeIsBusy(NovaException):
     message = _("deleting volume %(volume_name)s that has snapshot")
 
 
-class ExportDeviceNotFoundForVolume(NotFound):
-    message = _("No export device found for volume %(volume_id)s.")
+class SnapshotIsBusy(NovaException):
+    message = _("deleting snapshot %(snapshot_name)s that has "
+                "dependent volumes")
 
 
 class ISCSITargetNotFoundForVolume(NotFound):
@@ -393,6 +484,10 @@ class DiskNotFound(NotFound):
     message = _("No disk at %(location)s")
 
 
+class VolumeDriverNotFound(NotFound):
+    message = _("Could not find a handler for %(driver_type)s volume.")
+
+
 class InvalidImageRef(Invalid):
     message = _("Invalid image href %(image_href)s.")
 
@@ -467,6 +562,10 @@ class NetworkHostNotSet(NovaException):
     message = _("Host is not set to the network (%(network_id)s).")
 
 
+class NetworkBusy(NovaException):
+    message = _("Network %(network)s has active ports, cannot delete.")
+
+
 class DatastoreNotFound(NotFound):
     message = _("Could not find the datastore reference(s) which the VM uses.")
 
@@ -492,10 +591,6 @@ class FixedIpNotFoundForSpecificInstance(FixedIpNotFound):
     message = _("Instance %(instance_id)s doesn't have fixed ip '%(ip)s'.")
 
 
-class FixedIpNotFoundForVirtualInterface(FixedIpNotFound):
-    message = _("Virtual interface %(vif_id)s has zero associated fixed ips.")
-
-
 class FixedIpNotFoundForHost(FixedIpNotFound):
     message = _("Host %(host)s has zero fixed ips.")
 
@@ -525,6 +620,10 @@ class FloatingIpNotFound(NotFound):
     message = _("Floating ip not found for id %(id)s.")
 
 
+class FloatingIpDNSExists(Invalid):
+    message = _("The DNS entry %(name)s already exists in domain %(domain)s.")
+
+
 class FloatingIpNotFoundForAddress(FloatingIpNotFound):
     message = _("Floating ip not found for address %(address)s.")
 
@@ -537,14 +636,22 @@ class NoMoreFloatingIps(FloatingIpNotFound):
     message = _("Zero floating ips available.")
 
 
-class FloatingIpAlreadyInUse(NovaException):
-    message = _("Floating ip %(address)s already in use by %(fixed_ip)s.")
+class FloatingIpAssociated(NovaException):
+    message = _("Floating ip %(address)s is associated.")
+
+
+class FloatingIpNotAssociated(NovaException):
+    message = _("Floating ip %(address)s is not associated.")
 
 
 class NoFloatingIpsDefined(NotFound):
     message = _("Zero floating ips exist.")
 
 
+class NoFloatingIpInterface(NotFound):
+    message = _("Interface %(interface)s not found.")
+
+
 class KeypairNotFound(NotFound):
     message = _("Keypair %(name)s not found for user %(user_id)s")
 
@@ -600,12 +707,12 @@ class SecurityGroupNotFoundForRule(SecurityGroupNotFound):
 
 class SecurityGroupExistsForInstance(Invalid):
     message = _("Security group %(security_group_id)s is already associated"
-                 " with the instance %(instance_id)s")
+                " with the instance %(instance_id)s")
 
 
 class SecurityGroupNotExistsForInstance(Invalid):
     message = _("Security group %(security_group_id)s is not associated with"
-                 " the instance %(instance_id)s")
+                " the instance %(instance_id)s")
 
 
 class MigrationNotFound(NotFound):
@@ -640,6 +747,10 @@ class ConsoleNotFoundInPoolForInstance(ConsoleNotFound):
                 "in pool %(pool_id)s could not be found.")
 
 
+class ConsoleTypeInvalid(Invalid):
+    message = _("Invalid console type %(console_type)s ")
+
+
 class NoInstanceTypesFound(NotFound):
     message = _("Zero instance types found.")
 
@@ -657,8 +768,8 @@ class FlavorNotFound(NotFound):
     message = _("Flavor %(flavor_id)s could not be found.")
 
 
-class ZoneNotFound(NotFound):
-    message = _("Zone %(zone_id)s could not be found.")
+class CellNotFound(NotFound):
+    message = _("Cell %(cell_id)s could not be found.")
 
 
 class SchedulerHostFilterNotFound(NotFound):
@@ -771,6 +882,14 @@ class InstanceExists(Duplicate):
     message = _("Instance %(name)s already exists.")
 
 
+class InstanceTypeExists(Duplicate):
+    message = _("Instance Type %(name)s already exists.")
+
+
+class VolumeTypeExists(Duplicate):
+    message = _("Volume Type %(name)s already exists.")
+
+
 class InvalidSharedStorage(NovaException):
     message = _("%(path)s is on shared storage: %(reason)s")
 
@@ -783,44 +902,120 @@ class MalformedRequestBody(NovaException):
     message = _("Malformed message body: %(reason)s")
 
 
-class PasteConfigNotFound(NotFound):
-    message = _("Could not find paste config at %(path)s")
+class ConfigNotFound(NotFound):
+    message = _("Could not find config at %(path)s")
 
 
 class PasteAppNotFound(NotFound):
     message = _("Could not load paste app '%(name)s' from %(path)s")
 
 
-class VSANovaAccessParamNotFound(Invalid):
-    message = _("Nova access parameters were not specified.")
-
-
-class VirtualStorageArrayNotFound(NotFound):
-    message = _("Virtual Storage Array %(id)d could not be found.")
-
-
-class VirtualStorageArrayNotFoundByName(NotFound):
-    message = _("Virtual Storage Array %(name)s could not be found.")
-
-
 class CannotResizeToSameSize(NovaException):
     message = _("When resizing, instances must change size!")
 
 
-class CannotResizeToSmallerSize(NovaException):
-    message = _("Resizing to a smaller size is not supported.")
-
-
 class ImageTooLarge(NovaException):
     message = _("Image is larger than instance type allows")
 
 
-class ZoneRequestError(Error):
-    def __init__(self, message=None):
-        if message is None:
-            message = _("1 or more Zones could not complete the request")
-        super(ZoneRequestError, self).__init__(message=message)
+class ZoneRequestError(NovaException):
+    message = _("1 or more Zones could not complete the request")
+
+
+class InstanceTypeMemoryTooSmall(NovaException):
+    message = _("Instance type's memory is too small for requested image.")
+
+
+class InstanceTypeDiskTooSmall(NovaException):
+    message = _("Instance type's disk is too small for requested image.")
 
 
 class InsufficientFreeMemory(NovaException):
     message = _("Insufficient free memory on compute node to start %(uuid)s.")
+
+
+class CouldNotFetchMetrics(NovaException):
+    message = _("Could not fetch bandwidth/cpu/disk metrics for this host.")
+
+
+class NoValidHost(NovaException):
+    message = _("No valid host was found. %(reason)s")
+
+
+class WillNotSchedule(NovaException):
+    message = _("Host %(host)s is not up or doesn't exist.")
+
+
+class QuotaError(NovaException):
+    message = _("Quota exceeded") + ": code=%(code)s"
+
+
+class AggregateError(NovaException):
+    message = _("Aggregate %(aggregate_id)s: action '%(action)s' "
+                "caused an error: %(reason)s.")
+
+
+class AggregateNotFound(NotFound):
+    message = _("Aggregate %(aggregate_id)s could not be found.")
+
+
+class AggregateNameExists(Duplicate):
+    message = _("Aggregate %(aggregate_name)s already exists.")
+
+
+class AggregateHostNotFound(NotFound):
+    message = _("Aggregate %(aggregate_id)s has no host %(host)s.")
+
+
+class AggregateMetadataNotFound(NotFound):
+    message = _("Aggregate %(aggregate_id)s has no metadata with "
+                "key %(metadata_key)s.")
+
+
+class AggregateHostConflict(Duplicate):
+    message = _("Host %(host)s already member of another aggregate.")
+
+
+class AggregateHostExists(Duplicate):
+    message = _("Aggregate %(aggregate_id)s already has host %(host)s.")
+
+
+class DuplicateSfVolumeNames(Duplicate):
+    message = _("Detected more than one volume with name %(vol_name)s")
+
+
+class VolumeTypeCreateFailed(NovaException):
+    message = _("Cannot create volume_type with "
+                "name %(name)s and specs %(extra_specs)s")
+
+
+class InstanceTypeCreateFailed(NovaException):
+    message = _("Unable to create instance type")
+
+
+class SolidFireAPIException(NovaException):
+    message = _("Bad response from SolidFire API")
+
+
+class SolidFireAPIStatusException(SolidFireAPIException):
+    message = _("Error in SolidFire API response: status=%(status)s")
+
+
+class SolidFireAPIDataException(SolidFireAPIException):
+    message = _("Error in SolidFire API response: data=%(data)s")
+
+
+class DuplicateVlan(Duplicate):
+    message = _("Detected existing vlan with id %(vlan)d")
+
+
+class InstanceNotFound(NotFound):
+    message = _("Instance %(instance_id)s could not be found.")
+
+
+class InvalidInstanceIDMalformed(Invalid):
+    message = _("Invalid id: %(val)s (expecting \"i-...\").")
+
+
+class CouldNotFetchImage(NovaException):
+    message = _("Could not fetch image %(image)s")
