@@ -19,35 +19,45 @@
 """Quotas for instances, volumes, and floating ips."""
 
 from nova import db
-from nova import exception
+from nova.openstack.common import cfg
 from nova import flags
 
 
+quota_opts = [
+    cfg.IntOpt('quota_instances',
+               default=10,
+               help='number of instances allowed per project'),
+    cfg.IntOpt('quota_cores',
+               default=20,
+               help='number of instance cores allowed per project'),
+    cfg.IntOpt('quota_ram',
+               default=50 * 1024,
+               help='megabytes of instance ram allowed per project'),
+    cfg.IntOpt('quota_volumes',
+               default=10,
+               help='number of volumes allowed per project'),
+    cfg.IntOpt('quota_gigabytes',
+               default=1000,
+               help='number of volume gigabytes allowed per project'),
+    cfg.IntOpt('quota_floating_ips',
+               default=10,
+               help='number of floating ips allowed per project'),
+    cfg.IntOpt('quota_metadata_items',
+               default=128,
+               help='number of metadata items allowed per instance'),
+    cfg.IntOpt('quota_max_injected_files',
+               default=5,
+               help='number of injected files allowed'),
+    cfg.IntOpt('quota_max_injected_file_content_bytes',
+               default=10 * 1024,
+               help='number of bytes allowed per injected file'),
+    cfg.IntOpt('quota_max_injected_file_path_bytes',
+               default=255,
+               help='number of bytes allowed per injected file path'),
+    ]
+
 FLAGS = flags.FLAGS
-flags.DEFINE_integer('quota_instances', 10,
-                     'number of instances allowed per project')
-flags.DEFINE_integer('quota_cores', 20,
-                     'number of instance cores allowed per project')
-flags.DEFINE_integer('quota_ram', 50 * 1024,
-                     'megabytes of instance ram allowed per project')
-flags.DEFINE_integer('quota_volumes', 10,
-                     'number of volumes allowed per project')
-flags.DEFINE_integer('quota_gigabytes', 1000,
-                     'number of volume gigabytes allowed per project')
-flags.DEFINE_integer('quota_floating_ips', 10,
-                     'number of floating ips allowed per project')
-flags.DEFINE_integer('quota_metadata_items', 128,
-                     'number of metadata items allowed per instance')
-flags.DEFINE_integer('quota_max_injected_files', 5,
-                     'number of injected files allowed')
-flags.DEFINE_integer('quota_max_injected_file_content_bytes', 10 * 1024,
-                     'number of bytes allowed per injected file')
-flags.DEFINE_integer('quota_max_injected_file_path_bytes', 255,
-                     'number of bytes allowed per injected file path')
-flags.DEFINE_integer('quota_security_groups', 10,
-                     'number of security groups per project')
-flags.DEFINE_integer('quota_security_group_rules', 20,
-                     'number of security rules per security group')
+FLAGS.register_opts(quota_opts)
 
 
 def _get_default_quotas():
@@ -62,8 +72,6 @@ def _get_default_quotas():
         'injected_files': FLAGS.quota_max_injected_files,
         'injected_file_content_bytes':
             FLAGS.quota_max_injected_file_content_bytes,
-        'security_groups': FLAGS.quota_security_groups,
-        'security_group_rules': FLAGS.quota_security_group_rules,
     }
     # -1 in the quota flags means unlimited
     for key in defaults.keys():
@@ -102,9 +110,13 @@ def allowed_instances(context, requested_instances, instance_type):
     allowed_cores = _get_request_allotment(requested_cores, used_cores,
                                            quota['cores'])
     allowed_ram = _get_request_allotment(requested_ram, used_ram, quota['ram'])
-    allowed_instances = min(allowed_instances,
-                            allowed_cores // instance_type['vcpus'],
-                            allowed_ram // instance_type['memory_mb'])
+    if instance_type['vcpus']:
+        allowed_instances = min(allowed_instances,
+                                allowed_cores // instance_type['vcpus'])
+    if instance_type['memory_mb']:
+        allowed_instances = min(allowed_instances,
+                                allowed_ram // instance_type['memory_mb'])
+
     return min(requested_instances, allowed_instances)
 
 
@@ -140,32 +152,6 @@ def allowed_floating_ips(context, requested_floating_ips):
     return min(requested_floating_ips, allowed_floating_ips)
 
 
-def allowed_security_groups(context, requested_security_groups):
-    """Check quota and return min(requested, allowed) security groups."""
-    project_id = context.project_id
-    context = context.elevated()
-    used_sec_groups = db.security_group_count_by_project(context, project_id)
-    quota = get_project_quotas(context, project_id)
-    allowed_sec_groups = _get_request_allotment(requested_security_groups,
-                                                  used_sec_groups,
-                                                  quota['security_groups'])
-    return min(requested_security_groups, allowed_sec_groups)
-
-
-def allowed_security_group_rules(context, security_group_id,
-        requested_rules):
-    """Check quota and return min(requested, allowed) sec group rules."""
-    project_id = context.project_id
-    context = context.elevated()
-    used_rules = db.security_group_rule_count_by_group(context,
-                                                            security_group_id)
-    quota = get_project_quotas(context, project_id)
-    allowed_rules = _get_request_allotment(requested_rules,
-                                              used_rules,
-                                              quota['security_group_rules'])
-    return min(requested_rules, allowed_rules)
-
-
 def _calculate_simple_quota(context, resource, requested):
     """Check quota for resource; return min(requested, allowed)."""
     quota = get_project_quotas(context, context.project_id)
@@ -194,8 +180,3 @@ def allowed_injected_file_content_bytes(context, requested_bytes):
 def allowed_injected_file_path_bytes(context):
     """Return the number of bytes allowed in an injected file path."""
     return FLAGS.quota_max_injected_file_path_bytes
-
-
-class QuotaError(exception.ApiError):
-    """Quota Exceeded."""
-    pass
