@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright (c) 2010 Openstack, LLC.
+# Copyright (c) 2010 OpenStack, LLC.
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -22,23 +22,28 @@ Scheduler that allows routing some calls to one driver and others to another.
 """
 
 from nova import flags
+from nova.openstack.common import cfg
 from nova import utils
 from nova.scheduler import driver
 
 
-FLAGS = flags.FLAGS
-flags.DEFINE_string('compute_scheduler_driver',
-                    'nova.scheduler.chance.ChanceScheduler',
-                    'Driver to use for scheduling compute calls')
-flags.DEFINE_string('volume_scheduler_driver',
-                    'nova.scheduler.chance.ChanceScheduler',
-                    'Driver to use for scheduling volume calls')
+multi_scheduler_opts = [
+    cfg.StrOpt('compute_scheduler_driver',
+               default='nova.scheduler.'
+                    'filter_scheduler.FilterScheduler',
+               help='Driver to use for scheduling compute calls'),
+    cfg.StrOpt('volume_scheduler_driver',
+               default='nova.scheduler.chance.ChanceScheduler',
+               help='Driver to use for scheduling volume calls'),
+    ]
 
+FLAGS = flags.FLAGS
+FLAGS.register_opts(multi_scheduler_opts)
 
 # A mapping of methods to topics so we can figure out which driver to use.
-_METHOD_MAP = {'run_instance': 'compute',
-               'start_instance': 'compute',
-               'create_volume': 'volume'}
+# There are currently no compute methods proxied through the map
+_METHOD_MAP = {'create_volume': 'volume',
+               'create_volumes': 'volume'}
 
 
 class MultiScheduler(driver.Scheduler):
@@ -65,9 +70,12 @@ class MultiScheduler(driver.Scheduler):
             raise AttributeError(key)
         return getattr(self.drivers[_METHOD_MAP[method]], key)
 
-    def set_zone_manager(self, zone_manager):
-        for k, v in self.drivers.iteritems():
-            v.set_zone_manager(zone_manager)
+    def schedule(self, context, topic, method, *_args, **_kwargs):
+        return self.drivers[topic].schedule(context, topic,
+                method, *_args, **_kwargs)
 
-    def schedule(self, context, topic, *_args, **_kwargs):
-        return self.drivers[topic].schedule(context, topic, *_args, **_kwargs)
+    def schedule_run_instance(self, *args, **kwargs):
+        return self.drivers['compute'].schedule_run_instance(*args, **kwargs)
+
+    def schedule_prep_resize(self, *args, **kwargs):
+        return self.drivers['compute'].schedule_prep_resize(*args, **kwargs)
