@@ -17,7 +17,7 @@
 #    under the License.
 
 from nova import flags
+from nova.openstack.common import cfg
 
 FLAGS = flags.FLAGS
-
-flags.DEFINE_integer('runtime_answer', 54, 'test flag')
+FLAGS.register_opt(cfg.IntOpt('runtime_answer', default=54, help='test flag'))
