@@ -21,9 +21,12 @@ from nova import context
 from nova import db
 from nova import flags
 from nova import quota
+from nova import exception
+from nova import rpc
 from nova import test
 from nova import volume
 from nova.compute import instance_types
+from nova.scheduler import driver as scheduler_driver
 
 
 FLAGS = flags.FLAGS
@@ -44,20 +47,34 @@ class QuotaTestCase(test.TestCase):
                    quota_volumes=2,
                    quota_gigabytes=20,
                    quota_floating_ips=1,
-                   quota_security_groups=10,
-                   quota_security_group_rules=20)
+                   network_manager='nova.network.manager.FlatDHCPManager')
 
         self.network = self.network = self.start_service('network')
         self.user_id = 'admin'
         self.project_id = 'admin'
         self.context = context.RequestContext(self.user_id,
                                               self.project_id,
-                                              True)
+                                              is_admin=True)
+        orig_rpc_call = rpc.call
+
+        def rpc_call_wrapper(context, topic, msg):
+            """Stub out the scheduler creating the instance entry"""
+            if (topic == FLAGS.scheduler_topic and
+                msg['method'] == 'run_instance'):
+                scheduler = scheduler_driver.Scheduler
+                instance = scheduler().create_instance_db_entry(
+                        context,
+                        msg['args']['request_spec'])
+                return [scheduler_driver.encode_instance(instance)]
+            else:
+                return orig_rpc_call(context, topic, msg)
+
+        self.stubs.Set(rpc, 'call', rpc_call_wrapper)
 
     def _create_instance(self, cores=2):
         """Create a test instance"""
         inst = {}
-        inst['image_id'] = 1
+        inst['image_id'] = 'cedef40a-ed67-4d10-800e-17455edce175'
         inst['reservation_id'] = 'r-fakeres'
         inst['user_id'] = self.user_id
         inst['project_id'] = self.project_id
@@ -75,15 +92,25 @@ class QuotaTestCase(test.TestCase):
 
     def _get_instance_type(self, name):
         instance_types = {
-            'm1.tiny': dict(memory_mb=512, vcpus=1, local_gb=0, flavorid=1),
-            'm1.small': dict(memory_mb=2048, vcpus=1, local_gb=20, flavorid=2),
+            'm1.tiny': dict(memory_mb=512, vcpus=1, root_gb=0, flavorid=1),
+            'm1.small': dict(memory_mb=2048, vcpus=1, root_gb=20, flavorid=2),
             'm1.medium':
-                dict(memory_mb=4096, vcpus=2, local_gb=40, flavorid=3),
-            'm1.large': dict(memory_mb=8192, vcpus=4, local_gb=80, flavorid=4),
+                dict(memory_mb=4096, vcpus=2, root_gb=40, flavorid=3),
+            'm1.large': dict(memory_mb=8192, vcpus=4, root_gb=80, flavorid=4),
             'm1.xlarge':
-                dict(memory_mb=16384, vcpus=8, local_gb=160, flavorid=5)}
+                dict(memory_mb=16384, vcpus=8, root_gb=160, flavorid=5),
+            'm1.nocpu': dict(memory_mb=512, vcpus=0, root_gb=0, flavorid=6),
+            'm1.nomem': dict(memory_mb=0, vcpus=1, root_gb=0, flavorid=7)}
         return instance_types[name]
 
+    def test_quota_no_mem_no_cpu(self):
+        num_instances = quota.allowed_instances(self.context, 100,
+            self._get_instance_type('m1.nocpu'))
+        self.assertEqual(num_instances, 2)
+        num_instances = quota.allowed_instances(self.context, 100,
+            self._get_instance_type('m1.nomem'))
+        self.assertEqual(num_instances, 2)
+
     def test_quota_overrides(self):
         """Make sure overriding a projects quotas works"""
         num_instances = quota.allowed_instances(self.context, 100,
@@ -125,6 +152,10 @@ class QuotaTestCase(test.TestCase):
         num_instances = quota.allowed_instances(self.context, 100,
                                                 instance_type)
         self.assertEqual(num_instances, 100)
+        db.quota_create(self.context, self.project_id, 'instances', -1)
+        num_instances = quota.allowed_instances(self.context, 100,
+                                                instance_type)
+        self.assertEqual(num_instances, 100)
         num_instances = quota.allowed_instances(self.context, 101,
                                                 instance_type)
         self.assertEqual(num_instances, 101)
@@ -139,6 +170,10 @@ class QuotaTestCase(test.TestCase):
         num_instances = quota.allowed_instances(self.context, 100,
                                                 instance_type)
         self.assertEqual(num_instances, 100)
+        db.quota_create(self.context, self.project_id, 'ram', -1)
+        num_instances = quota.allowed_instances(self.context, 100,
+                                                instance_type)
+        self.assertEqual(num_instances, 100)
         num_instances = quota.allowed_instances(self.context, 101,
                                                 instance_type)
         self.assertEqual(num_instances, 101)
@@ -153,6 +188,10 @@ class QuotaTestCase(test.TestCase):
         num_instances = quota.allowed_instances(self.context, 100,
                                                 instance_type)
         self.assertEqual(num_instances, 100)
+        db.quota_create(self.context, self.project_id, 'cores', -1)
+        num_instances = quota.allowed_instances(self.context, 100,
+                                                instance_type)
+        self.assertEqual(num_instances, 100)
         num_instances = quota.allowed_instances(self.context, 101,
                                                 instance_type)
         self.assertEqual(num_instances, 101)
@@ -164,6 +203,9 @@ class QuotaTestCase(test.TestCase):
         db.quota_create(self.context, self.project_id, 'volumes', None)
         volumes = quota.allowed_volumes(self.context, 100, 1)
         self.assertEqual(volumes, 100)
+        db.quota_create(self.context, self.project_id, 'volumes', -1)
+        volumes = quota.allowed_volumes(self.context, 100, 1)
+        self.assertEqual(volumes, 100)
         volumes = quota.allowed_volumes(self.context, 101, 1)
         self.assertEqual(volumes, 101)
 
@@ -174,6 +216,9 @@ class QuotaTestCase(test.TestCase):
         db.quota_create(self.context, self.project_id, 'gigabytes', None)
         volumes = quota.allowed_volumes(self.context, 100, 1)
         self.assertEqual(volumes, 100)
+        db.quota_create(self.context, self.project_id, 'gigabytes', -1)
+        volumes = quota.allowed_volumes(self.context, 100, 1)
+        self.assertEqual(volumes, 100)
         volumes = quota.allowed_volumes(self.context, 101, 1)
         self.assertEqual(volumes, 101)
 
@@ -184,37 +229,12 @@ class QuotaTestCase(test.TestCase):
         db.quota_create(self.context, self.project_id, 'floating_ips', None)
         floating_ips = quota.allowed_floating_ips(self.context, 100)
         self.assertEqual(floating_ips, 100)
+        db.quota_create(self.context, self.project_id, 'floating_ips', -1)
+        floating_ips = quota.allowed_floating_ips(self.context, 100)
+        self.assertEqual(floating_ips, 100)
         floating_ips = quota.allowed_floating_ips(self.context, 101)
         self.assertEqual(floating_ips, 101)
 
-    def test_unlimited_security_groups(self):
-        self.flags(quota_security_groups=10)
-        security_groups = quota.allowed_security_groups(self.context, 100)
-        self.assertEqual(security_groups, 10)
-        db.quota_create(self.context, self.project_id, 'security_groups', None)
-        security_groups = quota.allowed_security_groups(self.context, 100)
-        self.assertEqual(security_groups, 100)
-        security_groups = quota.allowed_security_groups(self.context, 101)
-        self.assertEqual(security_groups, 101)
-
-    def test_unlimited_security_group_rules(self):
-
-        def fake_security_group_rule_count_by_group(context, sec_group_id):
-            return 0
-
-        self.stubs.Set(db, 'security_group_rule_count_by_group',
-                       fake_security_group_rule_count_by_group)
-
-        self.flags(quota_security_group_rules=20)
-        rules = quota.allowed_security_group_rules(self.context, 1234, 100)
-        self.assertEqual(rules, 20)
-        db.quota_create(self.context, self.project_id, 'security_group_rules',
-                        None)
-        rules = quota.allowed_security_group_rules(self.context, 1234, 100)
-        self.assertEqual(rules, 100)
-        rules = quota.allowed_security_group_rules(self.context, 1234, 101)
-        self.assertEqual(rules, 101)
-
     def test_unlimited_metadata_items(self):
         self.flags(quota_metadata_items=10)
         items = quota.allowed_metadata_items(self.context, 100)
@@ -222,6 +242,9 @@ class QuotaTestCase(test.TestCase):
         db.quota_create(self.context, self.project_id, 'metadata_items', None)
         items = quota.allowed_metadata_items(self.context, 100)
         self.assertEqual(items, 100)
+        db.quota_create(self.context, self.project_id, 'metadata_items', -1)
+        items = quota.allowed_metadata_items(self.context, 100)
+        self.assertEqual(items, 100)
         items = quota.allowed_metadata_items(self.context, 101)
         self.assertEqual(items, 101)
 
@@ -231,12 +254,13 @@ class QuotaTestCase(test.TestCase):
             instance_id = self._create_instance()
             instance_ids.append(instance_id)
         inst_type = instance_types.get_instance_type_by_name('m1.small')
-        self.assertRaises(quota.QuotaError, compute.API().create,
+        image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
+        self.assertRaises(exception.QuotaError, compute.API().create,
                                             self.context,
                                             min_count=1,
                                             max_count=1,
                                             instance_type=inst_type,
-                                            image_href=1)
+                                            image_href=image_uuid)
         for instance_id in instance_ids:
             db.instance_destroy(self.context, instance_id)
 
@@ -245,12 +269,13 @@ class QuotaTestCase(test.TestCase):
         instance_id = self._create_instance(cores=4)
         instance_ids.append(instance_id)
         inst_type = instance_types.get_instance_type_by_name('m1.small')
-        self.assertRaises(quota.QuotaError, compute.API().create,
+        image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
+        self.assertRaises(exception.QuotaError, compute.API().create,
                                             self.context,
                                             min_count=1,
                                             max_count=1,
                                             instance_type=inst_type,
-                                            image_href=1)
+                                            image_href=image_uuid)
         for instance_id in instance_ids:
             db.instance_destroy(self.context, instance_id)
 
@@ -259,13 +284,9 @@ class QuotaTestCase(test.TestCase):
         for i in range(FLAGS.quota_volumes):
             volume_id = self._create_volume()
             volume_ids.append(volume_id)
-        self.assertRaises(quota.QuotaError,
+        self.assertRaises(exception.QuotaError,
                           volume.API().create,
-                          self.context,
-                          size=10,
-                          snapshot_id=None,
-                          name='',
-                          description='')
+                          self.context, 10, '', '', None)
         for volume_id in volume_ids:
             db.volume_destroy(self.context, volume_id)
 
@@ -273,13 +294,9 @@ class QuotaTestCase(test.TestCase):
         volume_ids = []
         volume_id = self._create_volume(size=20)
         volume_ids.append(volume_id)
-        self.assertRaises(quota.QuotaError,
+        self.assertRaises(exception.QuotaError,
                           volume.API().create,
-                          self.context,
-                          size=10,
-                          snapshot_id=None,
-                          name='',
-                          description='')
+                          self.context, 10, '', '', None)
         for volume_id in volume_ids:
             db.volume_destroy(self.context, volume_id)
 
@@ -288,7 +305,7 @@ class QuotaTestCase(test.TestCase):
         db.floating_ip_create(context.get_admin_context(),
                               {'address': address,
                                'project_id': self.project_id})
-        self.assertRaises(quota.QuotaError,
+        self.assertRaises(exception.QuotaError,
                           self.network.allocate_floating_ip,
                           self.context,
                           self.project_id)
@@ -299,12 +316,13 @@ class QuotaTestCase(test.TestCase):
         for i in range(FLAGS.quota_metadata_items + 1):
             metadata['key%s' % i] = 'value%s' % i
         inst_type = instance_types.get_instance_type_by_name('m1.small')
-        self.assertRaises(quota.QuotaError, compute.API().create,
+        image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
+        self.assertRaises(exception.QuotaError, compute.API().create,
                                             self.context,
                                             min_count=1,
                                             max_count=1,
                                             instance_type=inst_type,
-                                            image_href='fake',
+                                            image_href=image_uuid,
                                             metadata=metadata)
 
     def test_default_allowed_injected_files(self):
@@ -353,15 +371,19 @@ class QuotaTestCase(test.TestCase):
         self.flags(image_service='nova.image.fake.FakeImageService')
         api = compute.API(image_service=self.StubImageService())
         inst_type = instance_types.get_instance_type_by_name('m1.small')
+        image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
         api.create(self.context, min_count=1, max_count=1,
-                instance_type=inst_type, image_href='3',
+                instance_type=inst_type, image_href=image_uuid,
                 injected_files=files)
 
     def test_no_injected_files(self):
         self.flags(image_service='nova.image.fake.FakeImageService')
         api = compute.API(image_service=self.StubImageService())
         inst_type = instance_types.get_instance_type_by_name('m1.small')
-        api.create(self.context, instance_type=inst_type, image_href='3')
+        image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
+        api.create(self.context,
+                   instance_type=inst_type,
+                   image_href=image_uuid)
 
     def test_max_injected_files(self):
         files = []
@@ -373,7 +395,7 @@ class QuotaTestCase(test.TestCase):
         files = []
         for i in xrange(FLAGS.quota_max_injected_files + 1):
             files.append(('/my/path%d' % i, 'my\ncontent%d\n' % i))
-        self.assertRaises(quota.QuotaError,
+        self.assertRaises(exception.QuotaError,
                           self._create_with_injected_files, files)
 
     def test_max_injected_file_content_bytes(self):
@@ -386,7 +408,7 @@ class QuotaTestCase(test.TestCase):
         max = FLAGS.quota_max_injected_file_content_bytes
         content = ''.join(['a' for i in xrange(max + 1)])
         files = [('/test/path', content)]
-        self.assertRaises(quota.QuotaError,
+        self.assertRaises(exception.QuotaError,
                           self._create_with_injected_files, files)
 
     def test_allowed_injected_file_path_bytes(self):
@@ -404,5 +426,5 @@ class QuotaTestCase(test.TestCase):
         max = FLAGS.quota_max_injected_file_path_bytes
         path = ''.join(['a' for i in xrange(max + 1)])
         files = [(path, 'config = quotatest')]
-        self.assertRaises(quota.QuotaError,
+        self.assertRaises(exception.QuotaError,
                           self._create_with_injected_files, files)
