@@ -21,9 +21,9 @@ Test suite for VMWareAPI.
 
 from nova import context
 from nova import db
+from nova import exception
 from nova import flags
 from nova import test
-from nova import utils
 from nova.compute import power_state
 from nova.tests.glance import stubs as glance_stubs
 from nova.tests.vmwareapi import db_fakes
@@ -40,14 +40,13 @@ class VMWareAPIVMTestCase(test.TestCase):
 
     def setUp(self):
         super(VMWareAPIVMTestCase, self).setUp()
-        self.context = context.RequestContext('fake', 'fake', False)
+        self.context = context.RequestContext('fake', 'fake', is_admin=False)
         self.flags(vmwareapi_host_ip='test_url',
                    vmwareapi_host_username='test_username',
                    vmwareapi_host_password='test_pass')
         self.user_id = 'fake'
         self.project_id = 'fake'
         self.context = context.RequestContext(self.user_id, self.project_id)
-        self.network = utils.import_object(FLAGS.network_manager)
         vmwareapi_fake.reset()
         db_fakes.stub_out_db_instance_api(self.stubs)
         stubs.set_stubs(self.stubs)
@@ -63,7 +62,7 @@ class VMWareAPIVMTestCase(test.TestCase):
                           {'broadcast': '192.168.0.255',
                            'dns': ['192.168.0.1'],
                            'gateway': '192.168.0.1',
-                           'gateway6': 'dead:beef::1',
+                           'gateway_v6': 'dead:beef::1',
                            'ip6s': [{'enabled': '1',
                                      'ip': 'dead:beef::dcad:beff:feef:0',
                                            'netmask': '64'}],
@@ -74,6 +73,12 @@ class VMWareAPIVMTestCase(test.TestCase):
                            'mac': 'DE:AD:BE:EF:00:00',
                            'rxtx_cap': 3})]
 
+        self.image = {
+            'id': 'c1c8ce3d-c2e0-4247-890c-ccf5cc1c004c',
+            'disk_format': 'vhd',
+            'size': 512,
+        }
+
     def tearDown(self):
         super(VMWareAPIVMTestCase, self).tearDown()
         vmwareapi_fake.cleanup()
@@ -95,7 +100,8 @@ class VMWareAPIVMTestCase(test.TestCase):
         """Create and spawn the VM."""
         self._create_instance_in_the_db()
         self.type_data = db.instance_type_get_by_name(None, 'm1.large')
-        self.conn.spawn(self.context, self.instance, self.network_info)
+        self.conn.spawn(self.context, self.instance, self.image,
+                        self.network_info)
         self._check_vm_record()
 
     def _check_vm_record(self):
@@ -107,7 +113,7 @@ class VMWareAPIVMTestCase(test.TestCase):
         self.assertEquals(len(instances), 1)
 
         # Get Nova record for VM
-        vm_info = self.conn.get_info(1)
+        vm_info = self.conn.get_info({'name': 1})
 
         # Get record for VM
         vms = vmwareapi_fake._get_objects("VirtualMachine")
@@ -150,87 +156,90 @@ class VMWareAPIVMTestCase(test.TestCase):
 
     def test_spawn(self):
         self._create_vm()
-        info = self.conn.get_info(1)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
 
     def test_snapshot(self):
         self._create_vm()
-        info = self.conn.get_info(1)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
         self.conn.snapshot(self.context, self.instance, "Test-Snapshot")
-        info = self.conn.get_info(1)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
 
     def test_snapshot_non_existent(self):
         self._create_instance_in_the_db()
-        self.assertRaises(Exception, self.conn.snapshot, self.context,
-                          self.instance, "Test-Snapshot")
+        self.assertRaises(exception.InstanceNotFound, self.conn.snapshot,
+                          self.context, self.instance, "Test-Snapshot")
 
     def test_reboot(self):
         self._create_vm()
-        info = self.conn.get_info(1)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
-        self.conn.reboot(self.instance, self.network_info)
-        info = self.conn.get_info(1)
+        reboot_type = "SOFT"
+        self.conn.reboot(self.instance, self.network_info, reboot_type)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
 
     def test_reboot_non_existent(self):
         self._create_instance_in_the_db()
-        self.assertRaises(Exception, self.conn.reboot, self.instance)
+        self.assertRaises(exception.InstanceNotFound, self.conn.reboot,
+                          self.instance, self.network_info, 'SOFT')
 
     def test_reboot_not_poweredon(self):
         self._create_vm()
-        info = self.conn.get_info(1)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
-        self.conn.suspend(self.instance, self.dummy_callback_handler)
-        info = self.conn.get_info(1)
+        self.conn.suspend(self.instance)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.PAUSED)
-        self.assertRaises(Exception, self.conn.reboot, self.instance)
+        self.assertRaises(exception.InstanceRebootFailure, self.conn.reboot,
+                          self.instance, self.network_info, 'SOFT')
 
     def test_suspend(self):
         self._create_vm()
-        info = self.conn.get_info(1)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
-        self.conn.suspend(self.instance, self.dummy_callback_handler)
-        info = self.conn.get_info(1)
+        self.conn.suspend(self.instance)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.PAUSED)
 
     def test_suspend_non_existent(self):
         self._create_instance_in_the_db()
-        self.assertRaises(Exception, self.conn.suspend, self.instance,
-                          self.dummy_callback_handler)
+        self.assertRaises(exception.InstanceNotFound, self.conn.suspend,
+                          self.instance)
 
     def test_resume(self):
         self._create_vm()
-        info = self.conn.get_info(1)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
-        self.conn.suspend(self.instance, self.dummy_callback_handler)
-        info = self.conn.get_info(1)
+        self.conn.suspend(self.instance)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.PAUSED)
-        self.conn.resume(self.instance, self.dummy_callback_handler)
-        info = self.conn.get_info(1)
+        self.conn.resume(self.instance)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
 
     def test_resume_non_existent(self):
         self._create_instance_in_the_db()
-        self.assertRaises(Exception, self.conn.resume, self.instance,
-                          self.dummy_callback_handler)
+        self.assertRaises(exception.InstanceNotFound, self.conn.resume,
+                          self.instance)
 
     def test_resume_not_suspended(self):
         self._create_vm()
-        info = self.conn.get_info(1)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
-        self.assertRaises(Exception, self.conn.resume, self.instance,
-                          self.dummy_callback_handler)
+        self.assertRaises(exception.InstanceResumeFailure, self.conn.resume,
+                          self.instance)
 
     def test_get_info(self):
         self._create_vm()
-        info = self.conn.get_info(1)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
 
     def test_destroy(self):
         self._create_vm()
-        info = self.conn.get_info(1)
+        info = self.conn.get_info({'name': 1})
         self._check_vm_info(info, power_state.RUNNING)
         instances = self.conn.list_instances()
         self.assertEquals(len(instances), 1)
@@ -254,12 +263,3 @@ class VMWareAPIVMTestCase(test.TestCase):
 
     def test_get_console_output(self):
         pass
-
-    def test_get_ajax_console(self):
-        pass
-
-    def dummy_callback_handler(self, ret):
-        """
-        Dummy callback function to be passed to suspend, resume, etc., calls.
-        """
-        pass
