@@ -22,9 +22,17 @@ Driver base-classes:
     types that support that contract
 """
 
+from nova import context as nova_context
+from nova import db
+from nova import flags
+from nova import log as logging
 from nova.compute import power_state
 
 
+LOG = logging.getLogger(__name__)
+FLAGS = flags.FLAGS
+
+
 class InstanceInfo(object):
     def __init__(self, name, state):
         self.name = name
@@ -87,9 +95,9 @@ class ComputeDriver(object):
     virtual network interface, and these IDs are opaque to the rest of Nova.
 
     Some methods here take an instance of nova.compute.service.Instance.  This
-    is the datastructure used by nova.compute to store details regarding an
+    is the data structure used by nova.compute to store details regarding an
     instance, and pass them into this layer.  This layer is responsible for
-    translating that generic datastructure into terms that are specific to the
+    translating that generic data structure into terms that are specific to the
     virtualization platform.
 
     """
@@ -100,7 +108,7 @@ class ComputeDriver(object):
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def get_info(self, instance_name):
+    def get_info(self, instance):
         """Get the current status of an instance, by name (not ID!)
 
         Returns a dict containing:
@@ -114,6 +122,38 @@ class ComputeDriver(object):
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
+    def get_num_instances(self):
+        """Return the total number of virtual machines.
+
+        Return the number of virtual machines that the hypervisor knows
+        about.
+
+        .. note::
+
+            This implementation works for all drivers, but it is
+            not particularly efficient. Maintainers of the virt drivers are
+            encouraged to override this method with something more
+            efficient.
+        """
+        return len(self.list_instances())
+
+    def instance_exists(self, instance_id):
+        """Checks existence of an instance on the host.
+
+        :param instance_id: The ID / name of the instance to lookup
+
+        Returns True if an instance with the supplied ID exists on
+        the host, False otherwise.
+
+        .. note::
+
+            This implementation works for all drivers, but it is
+            not particularly efficient. Maintainers of the virt drivers are
+            encouraged to override this method with something more
+            efficient.
+        """
+        return instance_id in self.list_instances()
+
     def list_instances(self):
         """
         Return the names of all the instances known to the virtualization
@@ -127,7 +167,7 @@ class ComputeDriver(object):
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def spawn(self, context, instance,
+    def spawn(self, context, instance, image_meta,
               network_info=None, block_device_info=None):
         """
         Create a new instance/VM/domain on the virtualization platform.
@@ -143,13 +183,16 @@ class ComputeDriver(object):
         :param instance: Instance object as returned by DB layer.
                          This function should use the data there to guide
                          the creation of the new instance.
+        :param image_meta: image object returned by nova.image.glance that
+                           defines the image from which to boot this instance
         :param network_info:
            :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
-        :param block_device_info:
+        :param block_device_info: Information about block devices to be
+                                  attached to the instance.
         """
         raise NotImplementedError()
 
-    def destroy(self, instance, network_info, cleanup=True):
+    def destroy(self, instance, network_info, block_device_info=None):
         """Destroy (shutdown and delete) the specified instance.
 
         If the instance is not found (for example if networking failed), this
@@ -159,18 +202,20 @@ class ComputeDriver(object):
         :param instance: Instance object as returned by DB layer.
         :param network_info:
            :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
-        :param cleanup:
+        :param block_device_info: Information about block devices that should
+                                  be detached from the instance.
 
         """
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def reboot(self, instance, network_info):
+    def reboot(self, instance, network_info, reboot_type):
         """Reboot the specified instance.
 
         :param instance: Instance object as returned by DB layer.
         :param network_info:
            :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
+        :param reboot_type: Either a HARD or SOFT reboot
         """
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
@@ -186,7 +231,7 @@ class ComputeDriver(object):
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def get_ajax_console(self, instance):
+    def get_vnc_console(self, instance):
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
@@ -195,6 +240,11 @@ class ComputeDriver(object):
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
+    def get_all_bw_usage(self, start_time, stop_time=None):
+        """Return bandwidth usage info for each interface on each
+           running VM"""
+        raise NotImplementedError()
+
     def get_host_ip_addr(self):
         """
         Retrieves the IP address of the dom0
@@ -202,12 +252,12 @@ class ComputeDriver(object):
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def attach_volume(self, context, instance_id, volume_id, mountpoint):
-        """Attach the disk at device_path to the instance at mountpoint"""
+    def attach_volume(self, connection_info, instance_name, mountpoint):
+        """Attach the disk to the instance at mountpoint using info"""
         raise NotImplementedError()
 
-    def detach_volume(self, context, instance_id, volume_id):
-        """Detach the disk attached to the instance at mountpoint"""
+    def detach_volume(self, connection_info, instance_name, mountpoint):
+        """Detach the disk attached to the instance"""
         raise NotImplementedError()
 
     def compare_cpu(self, cpu_info):
@@ -224,12 +274,12 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    def migrate_disk_and_power_off(self, instance, dest):
+    def migrate_disk_and_power_off(self, context, instance, dest,
+                                   instance_type, network_info):
         """
         Transfers the disk of a running instance in multiple phases, turning
         off the instance before the end.
         """
-        # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
     def snapshot(self, context, instance, image_id):
@@ -243,49 +293,65 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    def finish_migration(self, context, instance, disk_info, network_info,
-                         resize_instance):
+    def finish_migration(self, context, migration, instance, disk_info,
+                         network_info, image_meta, resize_instance):
         """Completes a resize, turning on the migrated instance
 
         :param network_info:
            :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
+        :param image_meta: image object returned by nova.image.glance that
+                           defines the image from which this instance
+                           was created
         """
         raise NotImplementedError()
 
-    def revert_migration(self, instance):
-        """Reverts a resize, powering back on the instance"""
+    def confirm_migration(self, migration, instance, network_info):
+        """Confirms a resize, destroying the source VM"""
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def pause(self, instance, callback):
+    def finish_revert_migration(self, instance, network_info):
+        """Finish reverting a resize, powering back on the instance"""
+        # TODO(Vek): Need to pass context in for access to auth_token
+        raise NotImplementedError()
+
+    def pause(self, instance):
         """Pause the specified instance."""
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def unpause(self, instance, callback):
+    def unpause(self, instance):
         """Unpause paused VM instance"""
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def suspend(self, instance, callback):
+    def suspend(self, instance):
         """suspend the specified instance"""
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def resume(self, instance, callback):
+    def resume(self, instance):
         """resume the specified instance"""
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def rescue(self, context, instance, callback, network_info):
+    def rescue(self, context, instance, network_info, image_meta):
         """Rescue the specified instance"""
         raise NotImplementedError()
 
-    def unrescue(self, instance, callback, network_info):
+    def unrescue(self, instance, network_info):
         """Unrescue the specified instance"""
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
+    def power_off(self, instance):
+        """Power off the specified instance."""
+        raise NotImplementedError()
+
+    def power_on(self, instance):
+        """Power on the specified instance"""
+        raise NotImplementedError()
+
     def update_available_resource(self, ctxt, host):
         """Updates compute manager resource info on ComputeNode table.
 
@@ -338,7 +404,7 @@ class ComputeDriver(object):
         This message is sent to the virtualization drivers on hosts that are
         running an instance that belongs to a security group that has a rule
         that references the security group identified by `security_group_id`.
-        It is the responsiblity of this method to make sure any rules
+        It is the responsibility of this method to make sure any rules
         that authorize traffic flow with members of the security group are
         updated and any new members can communicate, and any removed members
         cannot.
@@ -350,7 +416,7 @@ class ComputeDriver(object):
             * another host 'H1' runs an instance 'i-1'
             * instance 'i-1' is a member of security group 'b'
 
-            When 'i-1' launches or terminates we will recieve the message
+            When 'i-1' launches or terminates we will receive the message
             to update members of group 'b', at which time we will make
             any changes needed to the rules for instance 'i-0' to allow
             or deny traffic coming from 'i-1', depending on if it is being
@@ -360,7 +426,7 @@ class ComputeDriver(object):
         host 'H0' and this method would still have been called.  The point was
         that this method isn't called on the host where instances of that
         group are running (as is the case with
-        :method:`refresh_security_group_rules`) but is called where references
+        :py:meth:`refresh_security_group_rules`) but is called where references
         are made to authorizing those instances.
 
         An error should be raised if the operation cannot complete.
@@ -369,12 +435,12 @@ class ComputeDriver(object):
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
-    def refresh_provider_fw_rules(self, security_group_id):
+    def refresh_provider_fw_rules(self):
         """This triggers a firewall update based on database changes.
 
         When this is called, rules have either been added or removed from the
         datastore.  You can retrieve rules with
-        :method:`nova.db.api.provider_fw_rule_get_all`.
+        :py:meth:`nova.db.provider_fw_rule_get_all`.
 
         Provider rules take precedence over security group rules.  If an IP
         would be allowed by a security group ingress rule, but blocked by
@@ -463,25 +529,44 @@ class ComputeDriver(object):
         # TODO(Vek): Need to pass context in for access to auth_token
         pass
 
+    def poll_rebooting_instances(self, timeout):
+        """Poll for rebooting instances"""
+        # TODO(Vek): Need to pass context in for access to auth_token
+        raise NotImplementedError()
+
     def poll_rescued_instances(self, timeout):
         """Poll for rescued instances"""
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
+    def poll_unconfirmed_resizes(self, resize_confirm_window):
+        """Poll for unconfirmed resizes."""
+        # TODO(Vek): Need to pass context in for access to auth_token
+        raise NotImplementedError()
+
     def host_power_action(self, host, action):
         """Reboots, shuts down or powers up the host."""
         raise NotImplementedError()
 
+    def host_maintenance_mode(self, host, mode):
+        """Start/Stop host maintenance window. On start, it triggers
+        guest VMs evacuation."""
+        raise NotImplementedError()
+
     def set_host_enabled(self, host, enabled):
         """Sets the specified host's ability to accept new instances."""
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
     def plug_vifs(self, instance, network_info):
-        """Plugs in VIFs to networks."""
+        """Plug VIFs into networks."""
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
+    def unplug_vifs(self, instance, network_info):
+        """Unplug VIFs from networks."""
+        raise NotImplementedError()
+
     def update_host_status(self):
         """Refresh host stats"""
         raise NotImplementedError()
@@ -561,3 +646,42 @@ class ComputeDriver(object):
         Note that this function takes an instance ID.
         """
         raise NotImplementedError()
+
+    def legacy_nwinfo(self):
+        """
+        Indicate if the driver requires the legacy network_info format.
+        """
+        # TODO(tr3buchet): update all subclasses and remove this
+        return True
+
+    def manage_image_cache(self, context):
+        """
+        Manage the driver's local image cache.
+
+        Some drivers chose to cache images for instances on disk. This method
+        is an opportunity to do management of that cache which isn't directly
+        related to other calls into the driver. The prime example is to clean
+        the cache and remove images which are no longer of interest.
+        """
+
+    def add_to_aggregate(self, context, aggregate, host, **kwargs):
+        """Add a compute host to an aggregate."""
+        raise NotImplementedError()
+
+    def remove_from_aggregate(self, context, aggregate, host, **kwargs):
+        """Remove a compute host from an aggregate."""
+        raise NotImplementedError()
+
+    def get_volume_connector(self, instance):
+        """Get connector information for the instance for attaching to volumes.
+
+        Connector information is a dictionary representing the ip of the
+        machine that will be making the connection and and the name of the
+        iscsi initiator as follows::
+
+            {
+                'ip': ip,
+                'initiator': initiator,
+            }
+        """
+        raise NotImplementedError()
