@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2011 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -15,49 +17,38 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_log import log as logging
-from oslo_utils import importutils
-
-import nova.conf
 from nova import context
-from nova.network import linux_net
-from nova import objects
+from nova import db
+from nova import flags
+from nova import log as logging
+from nova.openstack.common import cfg
 from nova import utils
 from nova.virt import netutils
 
+
 LOG = logging.getLogger(__name__)
 
-CONF = nova.conf.CONF
+allow_same_net_traffic_opt = cfg.BoolOpt('allow_same_net_traffic',
+        default=True,
+        help='Whether to allow network traffic from same network')
 
-
-def load_driver(default, *args, **kwargs):
-    fw_class = importutils.import_class(CONF.firewall_driver or default)
-    return fw_class(*args, **kwargs)
+FLAGS = flags.FLAGS
+FLAGS.register_opt(allow_same_net_traffic_opt)
 
 
 class FirewallDriver(object):
-    """Firewall Driver base class.
-
-    Defines methods that any driver providing security groups should implement.
+    """ Firewall Driver base class.
 
+        Defines methods that any driver providing security groups
+        and provider fireall functionality should implement.
     """
     def prepare_instance_filter(self, instance, network_info):
         """Prepare filters for the instance.
-
-        At this point, the instance isn't running yet.
-        """
+        At this point, the instance isn't running yet."""
         raise NotImplementedError()
 
-    def filter_defer_apply_on(self):
-        """Defer application of IPTables rules."""
-        pass
-
-    def filter_defer_apply_off(self):
-        """Turn off deferral of IPTables rules and apply the rules now."""
-        pass
-
     def unfilter_instance(self, instance, network_info):
-        """Stop filtering instance."""
+        """Stop filtering instance"""
         raise NotImplementedError()
 
     def apply_instance_filter(self, instance, network_info):
@@ -74,16 +65,22 @@ class FirewallDriver(object):
         """Refresh security group rules from data store
 
         Gets called when a rule has been added to or removed from
-        the security group.
-        """
+        the security group."""
         raise NotImplementedError()
 
-    def refresh_instance_security_rules(self, instance):
-        """Refresh security group rules from data store
+    def refresh_security_group_members(self, security_group_id):
+        """Refresh security group members from data store
 
         Gets called when an instance gets added to or removed from
-        the security group the instance is a member of or if the
-        group gains or loses a rule.
+        the security group."""
+        raise NotImplementedError()
+
+    def refresh_provider_fw_rules(self):
+        """Refresh common rules for all hosts/instances from data store.
+
+        Gets called when a rule has been added to or removed from
+        the list of rules (via admin api).
+
         """
         raise NotImplementedError()
 
@@ -97,7 +94,7 @@ class FirewallDriver(object):
         raise NotImplementedError()
 
     def instance_filter_exists(self, instance, network_info):
-        """Check nova-instance-instance-xxx exists."""
+        """Check nova-instance-instance-xxx exists"""
         raise NotImplementedError()
 
 
@@ -105,12 +102,11 @@ class IptablesFirewallDriver(FirewallDriver):
     """Driver which enforces security groups through iptables rules."""
 
     def __init__(self, **kwargs):
+        from nova.network import linux_net
         self.iptables = linux_net.iptables_manager
-        self.instance_info = {}
-
-        # Flags for DHCP request rule
-        self.dhcp_create = False
-        self.dhcp_created = False
+        self.instances = {}
+        self.network_infos = {}
+        self.basicly_filtered = False
 
         self.iptables.ipv4['filter'].add_chain('sg-fallback')
         self.iptables.ipv4['filter'].add_rule('sg-fallback', '-j DROP')
@@ -124,68 +120,40 @@ class IptablesFirewallDriver(FirewallDriver):
         """No-op. Everything is done in prepare_instance_filter."""
         pass
 
-    def filter_defer_apply_on(self):
-        self.iptables.defer_apply_on()
-
-    def filter_defer_apply_off(self):
-        self.iptables.defer_apply_off()
-
     def unfilter_instance(self, instance, network_info):
-        if self.instance_info.pop(instance.id, None):
+        if self.instances.pop(instance['id'], None):
+            # NOTE(vish): use the passed info instead of the stored info
+            self.network_infos.pop(instance['id'])
             self.remove_filters_for_instance(instance)
             self.iptables.apply()
         else:
-            LOG.info('Attempted to unfilter instance which is not filtered',
-                     instance=instance)
+            LOG.info(_('Attempted to unfilter instance %s which is not '
+                     'filtered'), instance['id'])
 
     def prepare_instance_filter(self, instance, network_info):
-        self.instance_info[instance.id] = (instance, network_info)
-        ipv4_rules, ipv6_rules = self.instance_rules(instance, network_info)
-        self.add_filters_for_instance(instance, network_info, ipv4_rules,
-                                      ipv6_rules)
-        LOG.debug('Filters added to instance: %s', instance.id,
-                  instance=instance)
-        # Ensure that DHCP request rule is updated if necessary
-        if (self.dhcp_create and not self.dhcp_created):
-            self.iptables.ipv4['filter'].add_rule(
-                    'INPUT',
-                    '-s 0.0.0.0/32 -d 255.255.255.255/32 '
-                    '-p udp -m udp --sport 68 --dport 67 -j ACCEPT')
-            self.iptables.ipv4['filter'].add_rule(
-                    'FORWARD',
-                    '-s 0.0.0.0/32 -d 255.255.255.255/32 '
-                    '-p udp -m udp --sport 68 --dport 67 -j ACCEPT')
-            self.dhcp_created = True
+        self.instances[instance['id']] = instance
+        self.network_infos[instance['id']] = network_info
+        self.add_filters_for_instance(instance)
+        LOG.debug(_('Filters added to instance %s'), instance['uuid'])
+        self.refresh_provider_fw_rules()
+        LOG.debug(_('Provider Firewall Rules refreshed'))
         self.iptables.apply()
 
     def _create_filter(self, ips, chain_name):
         return ['-d %s -j $%s' % (ip, chain_name) for ip in ips]
 
-    def _get_subnets(self, network_info, version):
-        subnets = []
-        for vif in network_info:
-            if 'network' in vif and 'subnets' in vif['network']:
-                for subnet in vif['network']['subnets']:
-                    if subnet['version'] == version:
-                        subnets.append(subnet)
-        return subnets
-
     def _filters_for_instance(self, chain_name, network_info):
         """Creates a rule corresponding to each ip that defines a
-           jump to the corresponding instance - chain for all the traffic
-           destined to that ip.
-        """
-        v4_subnets = self._get_subnets(network_info, 4)
-        v6_subnets = self._get_subnets(network_info, 6)
-        ips_v4 = [ip['address'] for subnet in v4_subnets
-                                for ip in subnet['ips']]
+             jump to the corresponding instance - chain for all the traffic
+             destined to that ip."""
+        ips_v4 = [ip['ip'] for (_n, mapping) in network_info
+                 for ip in mapping['ips']]
         ipv4_rules = self._create_filter(ips_v4, chain_name)
 
-        ipv6_rules = ips_v6 = []
-        if CONF.use_ipv6:
-            if v6_subnets:
-                ips_v6 = [ip['address'] for subnet in v6_subnets
-                                        for ip in subnet['ips']]
+        ipv6_rules = []
+        if FLAGS.use_ipv6:
+            ips_v6 = [ip['ip'] for (_n, mapping) in network_info
+                     for ip in mapping['ip6s']]
             ipv6_rules = self._create_filter(ips_v6, chain_name)
 
         return ipv4_rules, ipv6_rules
@@ -194,30 +162,35 @@ class IptablesFirewallDriver(FirewallDriver):
         for rule in ipv4_rules:
             self.iptables.ipv4['filter'].add_rule(chain_name, rule)
 
-        if CONF.use_ipv6:
+        if FLAGS.use_ipv6:
             for rule in ipv6_rules:
                 self.iptables.ipv6['filter'].add_rule(chain_name, rule)
 
-    def add_filters_for_instance(self, instance, network_info, inst_ipv4_rules,
-                                 inst_ipv6_rules):
+    def add_filters_for_instance(self, instance):
+        network_info = self.network_infos[instance['id']]
         chain_name = self._instance_chain_name(instance)
-        if CONF.use_ipv6:
+        if FLAGS.use_ipv6:
             self.iptables.ipv6['filter'].add_chain(chain_name)
         self.iptables.ipv4['filter'].add_chain(chain_name)
         ipv4_rules, ipv6_rules = self._filters_for_instance(chain_name,
                                                             network_info)
         self._add_filters('local', ipv4_rules, ipv6_rules)
-        self._add_filters(chain_name, inst_ipv4_rules, inst_ipv6_rules)
+        ipv4_rules, ipv6_rules = self.instance_rules(instance, network_info)
+        self._add_filters(chain_name, ipv4_rules, ipv6_rules)
 
     def remove_filters_for_instance(self, instance):
         chain_name = self._instance_chain_name(instance)
 
         self.iptables.ipv4['filter'].remove_chain(chain_name)
-        if CONF.use_ipv6:
+        if FLAGS.use_ipv6:
             self.iptables.ipv6['filter'].remove_chain(chain_name)
 
+    @staticmethod
+    def _security_group_chain_name(security_group_id):
+        return 'nova-sg-%s' % (security_group_id,)
+
     def _instance_chain_name(self, instance):
-        return 'inst-%s' % (instance.id,)
+        return 'inst-%s' % (instance['id'],)
 
     def _do_basic_rules(self, ipv4_rules, ipv6_rules, network_info):
         # Always drop invalid packets
@@ -228,32 +201,31 @@ class IptablesFirewallDriver(FirewallDriver):
         ipv4_rules += ['-m state --state ESTABLISHED,RELATED -j ACCEPT']
         ipv6_rules += ['-m state --state ESTABLISHED,RELATED -j ACCEPT']
 
+        # Pass through provider-wide drops
+        ipv4_rules += ['-j $provider']
+        ipv6_rules += ['-j $provider']
+
     def _do_dhcp_rules(self, ipv4_rules, network_info):
-        v4_subnets = self._get_subnets(network_info, 4)
-        dhcp_servers = [subnet.get_meta('dhcp_server')
-            for subnet in v4_subnets if subnet.get_meta('dhcp_server')]
+        dhcp_servers = [info['dhcp_server'] for (_n, info) in network_info]
 
         for dhcp_server in dhcp_servers:
-            if dhcp_server:
-                ipv4_rules.append('-s %s -p udp --sport 67 --dport 68 '
-                                  '-j ACCEPT' % (dhcp_server,))
-                self.dhcp_create = True
+            ipv4_rules.append('-s %s -p udp --sport 67 --dport 68 '
+                              '-j ACCEPT' % (dhcp_server,))
 
     def _do_project_network_rules(self, ipv4_rules, ipv6_rules, network_info):
-        v4_subnets = self._get_subnets(network_info, 4)
-        v6_subnets = self._get_subnets(network_info, 6)
-        cidrs = [subnet['cidr'] for subnet in v4_subnets]
+        cidrs = [network['cidr'] for (network, _i) in network_info]
         for cidr in cidrs:
             ipv4_rules.append('-s %s -j ACCEPT' % (cidr,))
-        if CONF.use_ipv6:
-            cidrv6s = [subnet['cidr'] for subnet in v6_subnets]
+        if FLAGS.use_ipv6:
+            cidrv6s = [network['cidr_v6'] for (network, _i) in
+                       network_info]
+
             for cidrv6 in cidrv6s:
                 ipv6_rules.append('-s %s -j ACCEPT' % (cidrv6,))
 
     def _do_ra_rules(self, ipv6_rules, network_info):
-        v6_subnets = self._get_subnets(network_info, 6)
-        gateways_v6 = [subnet['gateway']['address'] for subnet in v6_subnets]
-
+        gateways_v6 = [mapping['gateway_v6'] for (_n, mapping) in
+                       network_info]
         for gateway_v6 in gateways_v6:
             ipv6_rules.append(
                     '-s %s/128 -p icmpv6 -j ACCEPT' % (gateway_v6,))
@@ -287,11 +259,6 @@ class IptablesFirewallDriver(FirewallDriver):
 
     def instance_rules(self, instance, network_info):
         ctxt = context.get_admin_context()
-        if isinstance(instance, dict):
-            # NOTE(danms): allow old-world instance objects from
-            # unconverted callers; all we need is instance.uuid below
-            instance = objects.Instance._from_db_object(
-                ctxt, objects.Instance(), instance, [])
 
         ipv4_rules = []
         ipv6_rules = []
@@ -301,125 +268,187 @@ class IptablesFirewallDriver(FirewallDriver):
         # Set up rules to allow traffic to/from DHCP server
         self._do_dhcp_rules(ipv4_rules, network_info)
 
-        # Allow project network traffic
-        if CONF.allow_same_net_traffic:
+        #Allow project network traffic
+        if FLAGS.allow_same_net_traffic:
             self._do_project_network_rules(ipv4_rules, ipv6_rules,
                                            network_info)
-        # We wrap these in CONF.use_ipv6 because they might cause
+        # We wrap these in FLAGS.use_ipv6 because they might cause
         # a DB lookup. The other ones are just list operations, so
         # they're not worth the clutter.
-        if CONF.use_ipv6:
+        if FLAGS.use_ipv6:
             # Allow RA responses
             self._do_ra_rules(ipv6_rules, network_info)
 
+        security_groups = db.security_group_get_by_instance(ctxt,
+                                                            instance['id'])
+
         # then, security group chains and rules
-        rules = objects.SecurityGroupRuleList.get_by_instance(ctxt, instance)
+        for security_group in security_groups:
+            rules = db.security_group_rule_get_by_security_group(ctxt,
+                                                          security_group['id'])
 
-        for rule in rules:
-            if not rule.cidr:
-                version = 4
-            else:
-                version = netutils.get_ip_version(rule.cidr)
+            for rule in rules:
+                LOG.debug(_('Adding security group rule: %r'), rule)
 
-            if version == 4:
-                fw_rules = ipv4_rules
-            else:
-                fw_rules = ipv6_rules
+                if not rule.cidr:
+                    version = 4
+                else:
+                    version = netutils.get_ip_version(rule.cidr)
 
-            protocol = rule.protocol
+                if version == 4:
+                    fw_rules = ipv4_rules
+                else:
+                    fw_rules = ipv6_rules
 
-            if protocol:
-                protocol = rule.protocol.lower()
+                protocol = rule.protocol
+                if version == 6 and rule.protocol == 'icmp':
+                    protocol = 'icmpv6'
 
-            if version == 6 and protocol == 'icmp':
-                protocol = 'icmpv6'
+                args = ['-j ACCEPT']
+                if protocol:
+                    args += ['-p', protocol]
 
-            args = ['-j ACCEPT']
-            if protocol:
-                args += ['-p', protocol]
+                if protocol in ['udp', 'tcp']:
+                    args += self._build_tcp_udp_rule(rule, version)
+                elif protocol == 'icmp':
+                    args += self._build_icmp_rule(rule, version)
+                if rule.cidr:
+                    LOG.info('Using cidr %r', rule.cidr)
+                    args += ['-s', rule.cidr]
+                    fw_rules += [' '.join(args)]
+                else:
+                    if rule['grantee_group']:
+                        # FIXME(jkoelker) This needs to be ported up into
+                        #                 the compute manager which already
+                        #                 has access to a nw_api handle,
+                        #                 and should be the only one making
+                        #                 making rpc calls.
+                        import nova.network
+                        nw_api = nova.network.API()
+                        for instance in rule['grantee_group']['instances']:
+                            LOG.info('instance: %r', instance)
+                            nw_info = nw_api.get_instance_nw_info(ctxt,
+                                                                  instance)
 
-            if protocol in ['udp', 'tcp']:
-                args += self._build_tcp_udp_rule(rule, version)
-            elif protocol == 'icmp':
-                args += self._build_icmp_rule(rule, version)
-            if rule.cidr:
-                args += ['-s', str(rule.cidr)]
-                fw_rules += [' '.join(args)]
-            else:
-                if rule.grantee_group:
-                    insts = objects.InstanceList.get_by_security_group(
-                            ctxt, rule.grantee_group)
-                    for inst in insts:
-                        if inst.info_cache.deleted:
-                            LOG.debug('ignoring deleted cache')
-                            continue
-                        nw_info = inst.get_network_info()
+                            ips = [ip['address']
+                                for ip in nw_info.fixed_ips()
+                                    if ip['version'] == version]
 
-                        ips = [ip['address'] for ip in nw_info.fixed_ips()
-                               if ip['version'] == version]
-
-                        LOG.debug('ips: %r', ips, instance=inst)
-                        for ip in ips:
-                            subrule = args + ['-s %s' % ip]
-                            fw_rules += [' '.join(subrule)]
+                            LOG.info('ips: %r', ips)
+                            for ip in ips:
+                                subrule = args + ['-s %s' % ip]
+                                fw_rules += [' '.join(subrule)]
 
+                LOG.info('Using fw_rules: %r', fw_rules)
         ipv4_rules += ['-j $sg-fallback']
         ipv6_rules += ['-j $sg-fallback']
-        LOG.debug('Security Group Rules %s translated to ipv4: %r, ipv6: %r',
-                  list(rules), ipv4_rules, ipv6_rules,
-                  instance=instance)
+
         return ipv4_rules, ipv6_rules
 
     def instance_filter_exists(self, instance, network_info):
         pass
 
+    def refresh_security_group_members(self, security_group):
+        self.do_refresh_security_group_rules(security_group)
+        self.iptables.apply()
+
     def refresh_security_group_rules(self, security_group):
         self.do_refresh_security_group_rules(security_group)
         self.iptables.apply()
 
-    def refresh_instance_security_rules(self, instance):
-        self.do_refresh_instance_rules(instance)
+    @utils.synchronized('iptables', external=True)
+    def do_refresh_security_group_rules(self, security_group):
+        for instance in self.instances.values():
+            self.remove_filters_for_instance(instance)
+            self.add_filters_for_instance(instance)
+
+    def refresh_provider_fw_rules(self):
+        """See :class:`FirewallDriver` docs."""
+        self._do_refresh_provider_fw_rules()
         self.iptables.apply()
 
     @utils.synchronized('iptables', external=True)
-    def _inner_do_refresh_rules(self, instance, network_info, ipv4_rules,
-                                ipv6_rules):
-        chain_name = self._instance_chain_name(instance)
-        if not self.iptables.ipv4['filter'].has_chain(chain_name):
-            LOG.info('instance chain %s disappeared during refresh, skipping',
-                     chain_name, instance=instance)
-            return
-        self.remove_filters_for_instance(instance)
-        self.add_filters_for_instance(instance, network_info, ipv4_rules,
-                                      ipv6_rules)
+    def _do_refresh_provider_fw_rules(self):
+        """Internal, synchronized version of refresh_provider_fw_rules."""
+        self._purge_provider_fw_rules()
+        self._build_provider_fw_rules()
 
-    def do_refresh_security_group_rules(self, security_group):
-        id_list = self.instance_info.keys()
-        for instance_id in id_list:
-            try:
-                instance, network_info = self.instance_info[instance_id]
-            except KeyError:
-                # NOTE(danms): instance cache must have been modified,
-                # ignore this deleted instance and move on
-                continue
-            ipv4_rules, ipv6_rules = self.instance_rules(instance,
-                                                         network_info)
-            self._inner_do_refresh_rules(instance, network_info, ipv4_rules,
-                                         ipv6_rules)
+    def _purge_provider_fw_rules(self):
+        """Remove all rules from the provider chains."""
+        self.iptables.ipv4['filter'].empty_chain('provider')
+        if FLAGS.use_ipv6:
+            self.iptables.ipv6['filter'].empty_chain('provider')
 
-    def do_refresh_instance_rules(self, instance):
-        _instance, network_info = self.instance_info[instance.id]
-        ipv4_rules, ipv6_rules = self.instance_rules(instance, network_info)
-        self._inner_do_refresh_rules(instance, network_info, ipv4_rules,
-                                     ipv6_rules)
+    def _build_provider_fw_rules(self):
+        """Create all rules for the provider IP DROPs."""
+        self.iptables.ipv4['filter'].add_chain('provider')
+        if FLAGS.use_ipv6:
+            self.iptables.ipv6['filter'].add_chain('provider')
+        ipv4_rules, ipv6_rules = self._provider_rules()
+        for rule in ipv4_rules:
+            self.iptables.ipv4['filter'].add_rule('provider', rule)
+
+        if FLAGS.use_ipv6:
+            for rule in ipv6_rules:
+                self.iptables.ipv6['filter'].add_rule('provider', rule)
+
+    @staticmethod
+    def _provider_rules():
+        """Generate a list of rules from provider for IP4 & IP6."""
+        ctxt = context.get_admin_context()
+        ipv4_rules = []
+        ipv6_rules = []
+        rules = db.provider_fw_rule_get_all(ctxt)
+        for rule in rules:
+            LOG.debug(_('Adding provider rule: %s'), rule['cidr'])
+            version = netutils.get_ip_version(rule['cidr'])
+            if version == 4:
+                fw_rules = ipv4_rules
+            else:
+                fw_rules = ipv6_rules
+
+            protocol = rule['protocol']
+            if version == 6 and protocol == 'icmp':
+                protocol = 'icmpv6'
+
+            args = ['-p', protocol, '-s', rule['cidr']]
+
+            if protocol in ['udp', 'tcp']:
+                if rule['from_port'] == rule['to_port']:
+                    args += ['--dport', '%s' % (rule['from_port'],)]
+                else:
+                    args += ['-m', 'multiport',
+                             '--dports', '%s:%s' % (rule['from_port'],
+                                                    rule['to_port'])]
+            elif protocol == 'icmp':
+                icmp_type = rule['from_port']
+                icmp_code = rule['to_port']
+
+                if icmp_type == -1:
+                    icmp_type_arg = None
+                else:
+                    icmp_type_arg = '%s' % icmp_type
+                    if not icmp_code == -1:
+                        icmp_type_arg += '/%s' % icmp_code
+
+                if icmp_type_arg:
+                    if version == 4:
+                        args += ['-m', 'icmp', '--icmp-type',
+                                 icmp_type_arg]
+                    elif version == 6:
+                        args += ['-m', 'icmp6', '--icmpv6-type',
+                                 icmp_type_arg]
+            args += ['-j DROP']
+            fw_rules += [' '.join(args)]
+        return ipv4_rules, ipv6_rules
 
 
 class NoopFirewallDriver(object):
     """Firewall driver which just provides No-op methods."""
-    def __init__(self, *args, **kwargs):
+    def __init__(*args, **kwargs):
         pass
 
-    def _noop(self, *args, **kwargs):
+    def _noop(*args, **kwargs):
         pass
 
     def __getattr__(self, key):
